@@ -12,7 +12,7 @@ namespace FSpot {
 		public abstract void Organize ();
 		
 		[DBus.Method]
-		public abstract void View (string path);
+		public abstract void View (string list);
 
 		[DBus.Method]
 		public abstract void Shutdown ();
@@ -24,11 +24,13 @@ namespace FSpot {
 		private static Db db;
 		static DBus.Connection connection;
 		System.Collections.ArrayList toplevels;
+		const string ServicePath = "org.gnome.FSpot";
+		const string CorePath = "/org/gnome/FSpot/Core";
 
 		public Core ()
 		{
 			toplevels = new System.Collections.ArrayList ();
-					
+			
 			// Load the database, upgrading/creating it as needed
 			string base_directory = FSpot.Global.BaseDirectory;
 			if (! File.Exists (base_directory))
@@ -53,16 +55,48 @@ namespace FSpot {
 
 		public static Core FindInstance ()
 		{
-			DBus.Service service = DBus.Service.Get (Connection, "org.gnome.FSpot");
-			return (Core)service.GetObject (typeof (Core), "/org/gnome/FSpot/Core");
+			DBus.Service service = DBus.Service.Get (Connection, ServicePath);
+			return (Core)service.GetObject (typeof (Core), CorePath);
+		}
+		
+		public void UnregisterServer ()
+		{
+			try { 
+				connection = null;
+				DBus.Service serv = DBus.Service.Get (Connection, ServicePath);
+				serv.UnregisterObject (this);
+			} catch (System.Exception e) {
+				// noop
+				System.Console.WriteLine ("unregister\n{0}\nunregister", e);
+			}
 		}
 
 		public void RegisterServer ()
 		{
-			DBus.Service service = new DBus.Service (Connection, "org.gnome.FSpot");
-			service.RegisterObject (this, "/org/gnome/FSpot/Core");
+			Service.RegisterObject (this, CorePath);
 		}
 		
+		public static void AssertOwnership () {
+			if (Service == null)
+				System.Console.WriteLine ("problems getting service");
+		}
+		
+		private static DBus.Service Service {
+			get {
+				DBus.Service service = null;
+
+				if (service == null) {
+					try {
+						service = DBus.Service.Get (Connection, ServicePath);
+					} catch (System.Exception e) {
+						//stem.Console.WriteLine (e);
+						service = new DBus.Service (Connection, ServicePath);
+					}
+				}				
+				return service;
+			}
+		}
+
 		private class ImportCommand 
 		{
 			string path;
@@ -108,20 +142,18 @@ namespace FSpot {
 			MainWindow.Window.Present ();
 		}
 		
-		public override void View (string path)
+		public override void View (string list)
 		{
-			if (System.IO.File.Exists (path) || System.IO.Directory.Exists (path))
-				Register (new FSpot.SingleView (path).Window);
-			else {
-				try {
-					System.Uri uri = new System.Uri (path);
-					path = uri.LocalPath;
-					if (System.IO.File.Exists (path) || System.IO.Directory.Exists (path))
-						Register (new FSpot.SingleView (path).Window);
-				} catch (System.Exception e) {
-					System.Console.WriteLine (e.ToString ());
-					System.Console.WriteLine ("no real valid path to view from {0}", path);
-				}
+			Viewbla (new UriList (list));
+		}
+
+		public void Viewbla (UriList list)
+		{
+			try {
+				Register (new FSpot.SingleView (list).Window);
+			} catch (System.Exception e) {
+				System.Console.WriteLine (e.ToString ());
+				System.Console.WriteLine ("no real valid path to view from {0}", list);
 			} 
 		}
 		
