@@ -3,6 +3,7 @@ using Gtk;
 using Gnome;
 using System.Collections;
 using System;
+using System.IO;
 
 public class ImportException : System.Exception {
 	public ImportException (string msg) : base (msg)
@@ -19,13 +20,36 @@ public class FileImportBackend : ImportBackend {
 
 	int count;
 
-	ArrayList file_paths;
-	ArrayList imported_photos;
+	ArrayList import_info;
+	Stack directories;
 
+	private class ImportInfo {
+		string original_path;
+		public string destination_path;
+		public Photo Photo;
+	       
+		public string OriginalPath {
+			get { return original_path; }
+		}
+		
+		public string DestinationPath {
+			get { return destination_path; }
+			set { destination_path = value; }
+		}
+		
+		public ImportInfo (string original)
+		{
+			original_path
+ = original;
+		        destination_path= null;
+			Photo = null;
+		}
+	}
+	
 	private void AddPath (string path)
 	{
 		if (FSpot.ImageFile.HasLoader (path))
-			file_paths.Add (path);
+			import_info.Add (new ImportInfo (path));
 	}
 
 	private void GetListing (System.IO.DirectoryInfo info)
@@ -39,12 +63,12 @@ public class FileImportBackend : ImportBackend {
 		}
 	}
 
-	private void GetListing (System.IO.DirectoryInfo info, System.IO.FileInfo [] files, bool recurse)
+	private void GetListing (System.IO.DirectoryInfo dirinfo, System.IO.FileInfo [] files, bool recurse)
 	{
-		System.Console.WriteLine ("Scanning {0}", info.FullName);
+		System.Console.WriteLine ("Scanning {0}", dirinfo.FullName);
 		Hashtable exiting_entries = new Hashtable ();
 
-		foreach (Photo p in store.Query (info)) {
+		foreach (Photo p in store.Query (dirinfo)) {
 			foreach (uint id in p.VersionIds) {
 				string name;
 				if (id == Photo.OriginalVersionId)
@@ -57,13 +81,13 @@ public class FileImportBackend : ImportBackend {
 		}
 	
 		foreach (System.IO.FileInfo f in files) {
-			if (exiting_entries [f.Name] == null) {
+			if (! exiting_entries.Contains (f.Name)) {
 				AddPath (f.FullName);
 			}
 		}
 
 		if (recurse) {
-			foreach (System.IO.DirectoryInfo d in info.GetDirectories ()){
+			foreach (System.IO.DirectoryInfo d in dirinfo.GetDirectories ()){
 				if (!d.Name.StartsWith ("."))
 					GetListing (d);
 			}
@@ -72,10 +96,10 @@ public class FileImportBackend : ImportBackend {
 
 	public override int Prepare ()
 	{
-		if (file_paths != null)
+		if (import_info != null)
 			throw new ImportException ("Busy");
 
-		file_paths = new ArrayList ();
+		import_info = new ArrayList ();
 
 		foreach (string path in base_paths) {
 			try {	
@@ -89,9 +113,8 @@ public class FileImportBackend : ImportBackend {
 			}
 		}	
 
-		imported_photos = new ArrayList ();
-
-		return file_paths.Count;
+		directories = new Stack ();
+		return import_info.Count;
 	}
 
 	public static string UniqueName (string path, string filename)
@@ -110,8 +133,13 @@ public class FileImportBackend : ImportBackend {
 		
 		return dest;
 	}
-
+	
 	public static string ChooseLocation (string path)
+	{
+		return ChooseLocation (path, null);
+	}
+
+	public static string ChooseLocation (string path, Stack created_directories)
 	{
 		string name = System.IO.Path.GetFileName (path);
 		FSpot.ImageFile img = FSpot.ImageFile.Create (path);
@@ -124,9 +152,34 @@ public class FileImportBackend : ImportBackend {
 						 time.Month,
 						 time.Day);
 		
-		if (!System.IO.Directory.Exists (dest_dir))
-			System.IO.Directory.CreateDirectory (dest_dir);
-		
+		if (!System.IO.Directory.Exists (dest_dir)) {
+			System.IO.DirectoryInfo info;
+			// Split dest_dir into constituent parts so we can clean up each individual directory in
+			// event of a cancel.
+			if (created_directories != null) {
+				string [] parts = dest_dir.Split (new char [] {'/'});
+				string nextPath = "";
+				for (int i = 0; i < parts.Length; i++) {
+					if (i == 0)
+						nextPath += parts [i];
+					else
+						nextPath += "/" + parts [i];
+					if (nextPath.Length > 0) {
+						info = new System.IO.DirectoryInfo (nextPath);
+						// only add the directory path if it didn't already exist and we haven't already added it.
+						if (!info.Exists && !created_directories.Contains (nextPath))
+							created_directories.Push (nextPath);
+					}
+				}
+			}
+			
+			info = System.IO.Directory.CreateDirectory (dest_dir);
+		}
+
+		// If the destination we'd like to use is the file itself return that
+		if (Path.Combine (dest_dir, name) == path)
+			return path;
+		 
 		string dest = UniqueName (dest_dir, name);
 		
 		return dest;
@@ -134,35 +187,51 @@ public class FileImportBackend : ImportBackend {
 
 	public override bool Step (out Photo photo, out Pixbuf thumbnail, out int count)
 	{
-		if (file_paths == null)
+		if (import_info == null)
 			throw new ImportException ("Prepare() was not called");
 
-		if (this.count == file_paths.Count)
+		if (this.count == import_info.Count)
 			throw new ImportException ("Already finished");
 
 		// FIXME Need to get the EXIF info etc.
-		string path = (string) file_paths [this.count];
-		
+		ImportInfo info = (ImportInfo)import_info [this.count];
+
 		try {
-			if (copy) {
-				string dest = ChooseLocation (path);
-				System.IO.File.Copy (path, dest);
-				photo = store.Create (dest, path, out thumbnail);
-				path = dest;
-			} else {
-                photo = store.Create (path, out thumbnail);
-			}
+			string destination = info.OriginalPath;
+			if (copy)
+				destination = ChooseLocation (info.OriginalPath, directories);
 			
+			// Don't copy if we are already home
+			if (info.OriginalPath == destination) {
+				info.DestinationPath = destination;
+				photo = store.Create (info.DestinationPath, out thumbnail);
+			} else {
+				System.IO.File.Copy (info.OriginalPath, destination);
+				info.DestinationPath = destination;
+
+				photo = store.Create (info.DestinationPath, info.OriginalPath, out thumbnail);
+
+				try {
+					File.SetAttributes (destination, File.GetAttributes (info.DestinationPath) & ~FileAttributes.ReadOnly);
+					DateTime create = File.GetCreationTime (info.OriginalPath);
+					File.SetCreationTime (info.DestinationPath, create);
+					DateTime mod = File.GetLastWriteTime (info.OriginalPath);
+					File.SetLastWriteTime (info.DestinationPath, mod);
+				} catch (System.Exception e) {
+					// we don't want an exception here to be fatal.
+				}
+			} 
+
 			if (tags != null) {
 				foreach (Tag t in tags) {
 					photo.AddTag (t);
 				}
 				store.Commit(photo);
 			}
-			imported_photos.Add (photo);
 			
+			info.Photo = photo;
 		} catch (System.Exception e) {
-			System.Console.WriteLine ("Error importing {0}\n{1}", path, e.ToString ());
+			System.Console.WriteLine ("Error importing {0}\n{1}", info.OriginalPath, e.ToString ());
 			thumbnail = null;
 			photo = null;
 		}
@@ -170,41 +239,53 @@ public class FileImportBackend : ImportBackend {
 		this.count ++;
 		count = this.count;
 
-		return count != file_paths.Count;
+		return count != import_info.Count;
 	}
 
 	public override void Cancel ()
 	{
-		if (imported_photos == null)
+		if (import_info == null)
 			throw new ImportException ("Not doing anything");
 
-		foreach (Photo p in imported_photos) {
-			if (copy) {
+		foreach (ImportInfo info in import_info) {
+			
+			if (info.OriginalPath != info.DestinationPath) {
 				try {
-					System.IO.File.Delete (p.DefaultVersionPath);
+					System.IO.File.Delete (info.DestinationPath);
 				} catch (System.Exception e) {
 					System.Console.WriteLine (e);
 				}
 			}
 			
-			store.Remove (p);
+			if (info.Photo != null)
+				store.Remove (info.Photo);
+		}
+		
+		// clean up all the directories we created.
+		if (copy) {
+			string path;
+			System.IO.DirectoryInfo info;
+			while (directories.Count > 0) {
+				path = directories.Pop () as string;
+				info = new System.IO.DirectoryInfo (path);
+				// double check we aren't trying to delete a directory that still contains something!
+				if (info.Exists && info.GetFiles().Length == 0 && info.GetDirectories().Length == 0)
+					info.Delete ();
+			}
 		}
-
-		Finish ();
 	}
 
 	public override void Finish ()
 	{
-		if (file_paths == null)
+		if (import_info == null)
 			throw new ImportException ("Not doing anything");
 
-		file_paths = null;
-
-		foreach (Photo p in imported_photos) {
-			FSpot.ThumbnailGenerator.Default.Request (p.DefaultVersionPath, 0, 256, 256);
+		foreach (ImportInfo info in import_info) {
+			if (info.Photo != null) 
+				FSpot.ThumbnailGenerator.Default.Request (info.Photo.DefaultVersionUri, 0, 256, 256);
 		}
 
-		imported_photos = null;
+		import_info = null;
 		count = 0;
 	}
 
