@@ -49,10 +49,8 @@ class FormClient {
 	
 	private void GenerateBoundary () 
 	{
-		// FIXME this shouldn't really be hardcoded, look in camel/camel-mime-utils.c
-		// for a boundary algo.  camel_header_msgid_generate ()
-
-		boundary = "--------ieoau._._+2_8_GoodLuck8.3-ds0d0J0S0Kl234324jfLdsjfdAuaoei-----";
+		Guid guid = Guid.NewGuid ();
+		boundary = "--------" + guid.ToString () + "-----";
 		start_boundary = "--" + boundary; 
 		end_boundary = start_boundary + "--";
 	}
@@ -117,11 +115,13 @@ class FormClient {
 					     + "{1}\r\n",
 					     MultipartHeader (name, value), value);
 		} else {
+			name = HttpUtility.UrlEncode (name.Replace(" ", "+"));
+			value = HttpUtility.UrlEncode (value.Replace(" ", "+"));
 			if (first_item) {
-				cmd = string.Format ("{0}={1}", name, HttpUtility.UrlEncode (value));
+				cmd = string.Format ("{0}={1}", name, value);
 				first_item = false;
 			} else {
-				cmd = string.Format ("&{0}={1}", name, HttpUtility.UrlEncode (value));
+				cmd = string.Format ("&{0}={1}", name, value);
 			}
 		}
 		//Console.WriteLine (cmd);
@@ -200,6 +200,18 @@ class FormClient {
 		Request = (HttpWebRequest) WebRequest.Create (uri);
 		CookieCollection cookie_collection = Cookies.GetCookies (uri);
 
+		if (uri.UserInfo != null && uri.UserInfo != "") {
+			NetworkCredential cred = new NetworkCredential ();
+			cred.GetCredential (uri, "basic");
+			CredentialCache credcache = new CredentialCache();
+			credcache.Add(uri, "basic", cred);
+			
+			Request.PreAuthenticate = true;
+			Request.Credentials = credcache;	
+		}
+
+		//System.Net.ServicePointManager.Expect100Continue = false;
+
 		Request.CookieContainer = new CookieContainer ();
 		foreach (Cookie c in cookie_collection) {
 			if (SuppressCookiePath) 
@@ -208,13 +220,14 @@ class FormClient {
 				Request.CookieContainer.Add (c);
 		}
 
-
 		Request.Method = "POST";
 		Request.Headers["Accept-Charset"] = "utf-8;";
 		
 		//Request.UserAgent = "F-Spot Gallery Remote Client";
 		Request.UserAgent = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.7) Gecko/20040626 Firefox/0.9.1";
 
+		Request.Proxy = WebProxy.GetDefaultProxy ();
+
 		if (multipart) {
 			GenerateBoundary ();
 			Request.ContentType = "multipart/form-data; boundary=" + boundary;
