@@ -17,24 +17,21 @@ namespace FSpot {
 				return System.IO.Path.Combine (HomeDirectory, "Photos");
 			}
 		}
-		
+
 		public static void ModifyColors (Gtk.Widget widget)
 		{
-#if false
-			Gdk.Color color = widget.Style.Background (Gtk.StateType.Normal);
-			color.Red = (ushort) (color.Red / 2);
-			color.Blue = (ushort) (color.Blue / 2);
-			color.Green = (ushort) (color.Green / 2);
-			widget.ModifyBg (Gtk.StateType.Normal, color);
-#else 
 			try {
 				widget.ModifyFg (Gtk.StateType.Normal, widget.Style.TextColors [(int)Gtk.StateType.Normal]);
+				widget.ModifyFg (Gtk.StateType.Active, widget.Style.TextColors [(int)Gtk.StateType.Active]);
+				widget.ModifyFg (Gtk.StateType.Selected, widget.Style.TextColors [(int)Gtk.StateType.Selected]);
 				widget.ModifyBg (Gtk.StateType.Normal, widget.Style.BaseColors [(int)Gtk.StateType.Normal]);
+				widget.ModifyBg (Gtk.StateType.Active, widget.Style.BaseColors [(int)Gtk.StateType.Active]);
+				widget.ModifyBg (Gtk.StateType.Selected, widget.Style.BaseColors [(int)Gtk.StateType.Selected]);
+				
 			} catch {
 				widget.ModifyFg (Gtk.StateType.Normal, widget.Style.Black);
-				widget.ModifyBg (Gtk.StateType.Normal, widget.Style.White);
+				widget.ModifyBg (Gtk.StateType.Normal, widget.Style.Black);
 			}
-#endif 
 		}
 
 		private static Cms.Profile display_profile;
