@@ -9,18 +9,35 @@ namespace FSpot {
 	}
 
 	public class ImageFile {
-		protected string path;
+		protected Uri uri;
 
 		static System.Collections.Hashtable name_table;
 
 		public ImageFile (string path) 
 		{
-			this.path = path;
+			this.uri = UriList.PathToFileUri (path);
 		}
 		
+		public ImageFile (Uri uri)
+		{
+			this.uri = uri;
+		}
+		
+		protected Stream Open ()
+		{
+			//Gnome.Vfs.Uri vfs = new Gnome.Vfs.Uri (uri.ToString ());
+			// FIXME, this seems like the sane thing to do, but vfs streams seem to 
+			// actually be faster and they need more testing.
+			//if (vfs.IsLocal)
+			//	return File.OpenRead (uri.LocalPath);
+
+			System.Console.WriteLine ("open uri = {0}", uri.ToString ());
+			return new Gnome.Vfs.VfsStream (uri.ToString (), FileMode.Open);
+		}
+
 		public virtual Stream PixbufStream ()
 		{
-			return File.OpenRead (this.path);
+			return Open ();
 		}
 
 		static ImageFile ()
@@ -47,23 +64,17 @@ namespace FSpot {
 			name_table [".x3f"] = typeof (FSpot.X3f.X3fFile);
 		}
 
-		public string Path {
-			get {
-				return this.path;
-			}
+		public Uri Uri {
+			get { return this.uri; }
 		}
 
 		public PixbufOrientation Orientation {
-			get {
-				return GetOrientation ();
-			}
+			get { return GetOrientation (); }
 		}
 
 		public virtual string Description
 		{
-			get { 
-				return null;
-			}
+			get { return null; }
 		}
 		
 		public virtual void Save (Gdk.Pixbuf pixbuf, System.IO.Stream stream)
@@ -86,8 +97,10 @@ namespace FSpot {
 		
 		public virtual Gdk.Pixbuf Load ()
 		{
-			Gdk.Pixbuf orig = new Gdk.Pixbuf (this.Path);
-			return TransformAndDispose (orig);
+			using (Stream stream = PixbufStream ()) {
+				Gdk.Pixbuf orig = new Gdk.Pixbuf (stream);
+				return TransformAndDispose (orig);
+			}
 		}
 		
 		public virtual Gdk.Pixbuf Load (int max_width, int max_height)
@@ -122,8 +135,10 @@ namespace FSpot {
 			get {
 				// FIXME mono uses the file change time (ctime) incorrectly
 				// as the creation time so we try to work around that slightly
-				DateTime create = File.GetCreationTimeUtc  (this.path);
-				DateTime write =  File.GetLastWriteTimeUtc (this.path);
+				Gnome.Vfs.FileInfo info = new Gnome.Vfs.FileInfo (uri.ToString ());
+
+				DateTime create = info.Ctime;
+				DateTime write = info.Mtime;
 
 				if (create < write)
 					return create;
@@ -134,6 +149,12 @@ namespace FSpot {
 
 		public static bool HasLoader (string path)
 		{
+			return HasLoader (UriList.PathToFileUri (path));
+		}
+		
+		public static bool HasLoader (Uri uri)
+		{
+			string path = uri.AbsolutePath;
 			string extension = System.IO.Path.GetExtension (path).ToLower ();
 			System.Type t = (System.Type) name_table [extension];
 			
@@ -142,16 +163,36 @@ namespace FSpot {
 
 		public static ImageFile Create (string path)
 		{
+			return Create (UriList.PathToFileUri (path));
+		}
+
+		public static ImageFile Create (Uri uri)
+		{
+			string path = uri.AbsolutePath;
 			string extension = System.IO.Path.GetExtension (path).ToLower ();
 			System.Type t = (System.Type) name_table [extension];
-			
 			ImageFile img;
+
 			if (t != null)
-				img = (ImageFile) System.Activator.CreateInstance (t, new object[] {path});
+				img = (ImageFile) System.Activator.CreateInstance (t, new object[] { uri });
 			else 
-				img = new ImageFile (path);
+				img = new ImageFile (uri);
 
 			return img;
 		}
+		
+		// FIXME this is a horrible hack to get a temporary name
+		// with the right extension for ImageFile until we use the mime data
+		// properly.  It is here to make sure we can find the places that use
+		// this hack
+		public static string TempPath (string name)
+		{
+			string temp = System.IO.Path.GetTempFileName ();
+			string imgtemp = temp + System.IO.Path.GetExtension (name);
+
+			System.IO.File.Move (temp, imgtemp);
+
+			return imgtemp;
+		}
 	} 
 }
