@@ -319,6 +319,8 @@ public class ImportCommand : FSpot.GladeDialog {
 	ImportBackend importer;
 	IconView tray;
 
+	FSpot.Delay idle_start; 
+
 	string loading_string;
 
 	string import_path;
@@ -363,14 +365,15 @@ public class ImportCommand : FSpot.GladeDialog {
 				MainWindow.Toplevel.ImportCamera (port);
 			}
 
-			Start ();
+			idle_start.Start ();
 		}
 	}
 
 	public ImportCommand (Gtk.Window mw)
 	{
 		main_window = mw;
-		step = new FSpot.Delay (10, new GLib.IdleHandler (Step));
+		step = new FSpot.Delay (new GLib.IdleHandler (Step));
+		idle_start = new FSpot.Delay (new IdleHandler (Start));
 		loading_string = Catalog.GetString ("Loading {0} of {1}");
 	}
 
@@ -414,6 +417,7 @@ public class ImportCommand : FSpot.GladeDialog {
 			// with the recursive DoImport loops we sometimes get into
 			ongoing = importer.Step (out photo, out thumbnail, out count);
 		} catch (ImportException e){
+			System.Console.WriteLine (e);
 			return false;
 		}
 
@@ -424,16 +428,20 @@ public class ImportCommand : FSpot.GladeDialog {
 			collection.Add (photo);
 		
 			//grid.AddThumbnail (thumbnail);
-			if (count < total)
-				UpdateProgressBar (count + 1, total);
 
 			thumbnail.Dispose ();
 		}
+		if (count < total)
+			UpdateProgressBar (count + 1, total);
 
 		if (ongoing && total > 0)
 			return true;
 		else {
 			System.Console.WriteLine ("Stopping");
+			if (progress_bar != null)
+				progress_bar.Text = Catalog.GetString ("Done Loading");
+			
+			AllowFinish = true;
 			return false;
 		}
 	}
@@ -462,8 +470,6 @@ public class ImportCommand : FSpot.GladeDialog {
 		collection.Clear ();
 		collection.Capacity = total;
 
-		FSpot.ThumbnailGenerator.Default.PushBlock ();
-
 		while (total > 0 && this.Step ()) {
 			System.DateTime start_time = System.DateTime.Now;
 			System.TimeSpan span = start_time - start_time;
@@ -474,23 +480,20 @@ public class ImportCommand : FSpot.GladeDialog {
 			}
 		}
 
-		FSpot.ThumbnailGenerator.Default.PopBlock ();
-
-		if (progress_bar != null)
-			progress_bar.Text = Catalog.GetString ("Done Loading");
-
-		AllowFinish = true;
-
 		return total;
 	}
 	
 	public void Finish ()
 	{
+		if (idle_start.IsPending || step.IsPending) {
+			AllowFinish = false;
+			return;
+		}
+		
 		if (importer != null)
 			importer.Finish ();
 		
 		importer = null;
-
 	}
 	
 	public void HandleTagToggled (object o, EventArgs args) 
@@ -543,13 +546,12 @@ public class ImportCommand : FSpot.GladeDialog {
 		//tag_label.Text = t.Name;
 	}
 
-
 	private void HandleRecurseToggled (object sender, System.EventArgs args)
 	{
 		this.Cancel ();
 		this.Dialog.Sensitive = false;
 	       
-		Idle.Add (new IdleHandler (Start));
+		idle_start.Start ();
 	}
 
 	public int ImportFromFile (PhotoStore store, string path)
@@ -615,13 +617,19 @@ public class ImportCommand : FSpot.GladeDialog {
 				SetImportPath (path);
 				source_option_menu.SetHistory (0);
 			} 
+			idle_start.Start ();
 		}
 						
 		ResponseType response = (ResponseType) this.Dialog.Run ();
 		
 		while (response == ResponseType.Ok) {
-			if (Directory.Exists (this.ImportPath))
+			try {
+				if (Directory.Exists (this.ImportPath))
+					break;
+			} catch (System.Exception e){
+				System.Console.WriteLine (e);
 				break;
+			}
 
 			HigMessageDialog md = new HigMessageDialog (this.Dialog,
 			        DialogFlags.DestroyWithParent,
@@ -678,16 +686,19 @@ public class ImportCommand : FSpot.GladeDialog {
 
 	private void HandleNewTagSelected (object sender, EventArgs args)
 	{
-		Tag new_tag = MainWindow.Toplevel.CreateTag ();
+		Tag new_tag = MainWindow.Toplevel.CreateTag (Dialog, null);
 
 		if (new_tag != null) {
 			CreateTagMenu ();
 			tag_option_menu.SetHistory ((uint) (tag_option_menu.Menu as TagMenu).GetPosition (new_tag));
+			tag_selected = new_tag;
 		}
 	}
 
 	public void Cancel ()
 	{
+		idle_start.Stop ();
+		step.Stop ();
 		if (importer != null) {
 			importer.Cancel ();
 			importer = null;
@@ -714,22 +725,30 @@ public class ImportCommand : FSpot.GladeDialog {
 		string [] pathimport =  {ImportPath};
 		//this.Dialog.Destroy();
 		
+		if (copy_check != null)
+			copy = copy_check.Active;
+		
 		bool recurse = true;
 		if (recurse_check != null)
 			recurse = recurse_check.Active;
 		
-		if (collection == null)
-			return false;
+		importer = new FileImportBackend (store, pathimport, copy, recurse, null);
+		AllowFinish = false;
+		
+		total = importer.Prepare ();
+		
+		if (total > 0)
+			UpdateProgressBar (1, total);
+		
+		collection.Clear ();
+		collection.Capacity = total;
 
-		DoImport (new FileImportBackend (store, pathimport, copy, recurse, null));
+		if (total > 0)
+			step.Start ();
+	       
 		return false;
 	}
 
-	public int ImportFromPaths (PhotoStore store, string [] paths)
-	{
-		return ImportFromPaths (store, paths, null);
-	}
-
 	public int ImportFromPaths (PhotoStore store, string [] paths, bool copy)
 	{
 		return ImportFromPaths (store, paths, null, copy);
