@@ -8,12 +8,12 @@ using SemWeb;
 // gracefully using ellipsis.
 
 public class InfoBox : VBox {
-
+	Delay update_delay;
 	private Photo photo;
 	public Photo Photo {
 		set {
 			photo = value;
-			Update ();
+			update_delay.Start ();
 		}
 
 		get {
@@ -124,8 +124,9 @@ public class InfoBox : VBox {
 
 		MemoryStore store;
 		
+#if USE_EXIF_DATE
 		DateTime date;
-
+#endif
 		public ImageInfo (ImageFile img) 
 		{
 			// FIXME We use the memory store to hold the anonymous statements
@@ -152,12 +153,13 @@ public class InfoBox : VBox {
 				int real_width;
 				int real_height;
 
-				JpegUtils.GetSize (img.Path, out real_width, out real_height);
+				JpegUtils.GetSize (img.Uri.LocalPath, out real_width, out real_height);
 				width = real_width.ToString ();
 				height = real_height.ToString ();
 			}
-
-			date = img.Date;
+#if USE_EXIF_DATE
+			date = img.Date.ToLocalTime ();
+#endif
 		}
 
 		public bool Add (SemWeb.Statement stmt)
@@ -225,7 +227,7 @@ public class InfoBox : VBox {
 					return Mono.Posix.Catalog.GetString ("(Unknown)");
 			}
 		}
-
+#if USE_EXIF_DATE
 		public string Date {
 			get {
 				if (date > DateTime.MinValue && date < DateTime.MaxValue)
@@ -234,27 +236,26 @@ public class InfoBox : VBox {
 					return Mono.Posix.Catalog.GetString ("(Unknown)");
 			}
 		}
+#endif
 	}
 		
 
-	public void Update ()
+	public bool Update ()
 	{
 		ImageFile img;
 		ImageInfo info;
 
 		if (photo == null) {
 			Clear ();
-			return;
+			return false;
 		}
 		
-		
-		string path = photo.DefaultVersionUri.LocalPath;
-		name_entry.Text = System.IO.Path.GetFileName (path);
+		name_entry.Text = photo.Name;
 		try {
-			using (new Timer ("building info")) {
-				img = ImageFile.Create (path);
+			//using (new Timer ("building info")) {
+				img = ImageFile.Create (photo.DefaultVersionUri);
 				info = new ImageInfo (img);
-			}
+				//}
 		} catch (System.Exception e) {
 			System.Console.WriteLine (e);
 			info = new ImageInfo (null);			
@@ -264,8 +265,13 @@ public class InfoBox : VBox {
 		name_entry.Sensitive = true;
 		exposure_info_label.Text = info.ExposureInfo;
 		size_label.Text = info.Dimensions;
+#if USE_EXIF_DATE
 		date_label.Text = info.Date;
-
+#else
+		date_label.Text = String.Format ("{0}\n{1}",
+						 photo.Time.ToLocalTime ().ToShortDateString (),
+						 photo.Time.ToLocalTime ().ToShortTimeString ());
+#endif
 		version_option_menu.Sensitive = true;
 		PhotoVersionMenu menu = new PhotoVersionMenu (photo);
 		menu.VersionIdChanged += new PhotoVersionMenu.VersionIdChangedHandler (HandleVersionIdChanged);
@@ -281,6 +287,8 @@ public class InfoBox : VBox {
 			}
 			i++;
 		}
+
+		return false;
 	}
 
 
@@ -289,7 +297,8 @@ public class InfoBox : VBox {
 	public InfoBox () : base (false, 0)
 	{
 		SetupWidgets ();
-		Update ();
+		update_delay = new Delay (Update);
+		update_delay.Start ();
 
 		BorderWidth = 6;
 	}
