@@ -1,3 +1,22 @@
+/*
+ * Copyright (c) 2006 Novell Inc. 
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public
+ * License along with this program; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ */
+
 using System;
 using System.IO;
 using FSpot.Xmp;
@@ -89,6 +108,12 @@ public class JpegHeader : SemWeb.StatementSource {
 	private System.Collections.ArrayList marker_list = new System.Collections.ArrayList ();	
 	private byte [] image_data;
 
+                // False seems a safe default
+                public bool Distinct {
+                        get { return false; }
+                }
+
+
 	public class Marker {
 		public JpegMarker Type;
 		public byte [] Data;
@@ -129,8 +154,8 @@ public class JpegHeader : SemWeb.StatementSource {
 			
 			int j;
 			for (j = 0; j < this.Data.Length; j++) {
-					if (this.Data [j] == 0x00)
-						break;
+				if (this.Data [j] == 0x00)
+					break;
 					
 			}
 			
@@ -139,8 +164,21 @@ public class JpegHeader : SemWeb.StatementSource {
 			else 
 				return null;
 		}
+		
+		private static int Read (Stream stream, byte [] dest, int start, int len)
+		{
+			int pos = 0;
 
-		public static Marker Load (System.IO.Stream stream) {
+			while (pos < len) {
+				int read = stream.Read (dest, pos + start, len - pos);
+				if (read < 0)
+					break;
+				pos += read;
+			}
+			return pos;
+		}
+
+		public static Marker Load (Stream stream) {
 			byte [] raw = new byte [2];
 			ushort length;
 		       
@@ -149,11 +187,11 @@ public class JpegHeader : SemWeb.StatementSource {
 
 			// FIXME there is a potential loop here.
 			
-			raw [0] = (byte)stream.ReadByte ();
-			if (raw [0] != 0xff)
+			int read = Read (stream, raw, 0, 2);
+			if (read < 2 && raw [0] != 0xff)
 				throw new System.Exception (System.String.Format ("Invalid marker found {0}", raw [0]));
 			
-			JpegMarker id = (JpegMarker) stream.ReadByte ();
+			JpegMarker id = (JpegMarker) raw [1];
 			switch (id) {
 			case JpegMarker.Soi:
 			case JpegMarker.Eoi:
@@ -169,11 +207,11 @@ public class JpegHeader : SemWeb.StatementSource {
 			case (JpegMarker) 0:
 				return new Marker (id, null);
 			default:
-				stream.Read (raw, 0, 2);
+				Read (stream, raw, 0, 2);
 				length = FSpot.BitConverter.ToUInt16 (raw, 0, false);
 				
 				byte [] data = new byte [length - 2];
-				stream.Read (data, 0, data.Length);
+				Read (stream, data, 0, data.Length);
 				return new Marker (id, data);
 			}
 			
@@ -211,6 +249,7 @@ public class JpegHeader : SemWeb.StatementSource {
 	public static Signature ExifSignature = new Signature (JpegMarker.App1, "Exif\0\0");
 	public static Signature IccProfileSignature = new Signature (JpegMarker.App2, "ICC_PROFILE\0");
 	public static Signature PhotoshopSignature = new Signature (JpegMarker.App13, "Photoshop 3.0\0");
+	public static Signature QuantizationSignature = new Signature (JpegMarker.Dqt, null);
 
 	public class Signature {
 		public JpegMarker Id;
@@ -264,7 +303,7 @@ public class JpegHeader : SemWeb.StatementSource {
 		return null;
 	}
 	
-	private FSpot.Tiff.Header GetExifHeader ()
+	public FSpot.Tiff.Header GetExifHeader ()
 	{
 		string name = ExifSignature.Name;
 		Marker marker = FindMarker (ExifSignature);
@@ -272,9 +311,10 @@ public class JpegHeader : SemWeb.StatementSource {
 		if (marker == null)
 			return null;
 		
-		System.IO.Stream exifstream = new System.IO.MemoryStream (marker.Data, name.Length, marker.Data.Length - name.Length, false);
-		FSpot.Tiff.Header exif = new FSpot.Tiff.Header (exifstream);
-		return exif;
+		using (System.IO.Stream exifstream = new System.IO.MemoryStream (marker.Data, name.Length, marker.Data.Length - name.Length, false)) {
+			FSpot.Tiff.Header exif = new FSpot.Tiff.Header (exifstream);
+			return exif;
+		}
 	}
 
 	public XmpFile GetXmp ()
@@ -285,11 +325,12 @@ public class JpegHeader : SemWeb.StatementSource {
 			int len = name.Length;
 			//System.Console.WriteLine (System.Text.Encoding.ASCII.GetString (marker.Data, len, 
 			//								marker.Data.Length - len));
-			System.IO.Stream xmpstream = new System.IO.MemoryStream (marker.Data, len, 
-										 marker.Data.Length - len, false);
+			using (System.IO.Stream xmpstream = new System.IO.MemoryStream (marker.Data, len, 
+											marker.Data.Length - len, false)) {
 			
-			XmpFile xmp = new XmpFile (xmpstream);					
-			return xmp;
+				XmpFile xmp = new XmpFile (xmpstream);					
+				return xmp;
+			}
 		}
 		return null;
 	}
@@ -308,9 +349,10 @@ public class JpegHeader : SemWeb.StatementSource {
 		JpegHeader.Marker marker = FindMarker (PhotoshopSignature);
 		if (marker != null) {
 			int len = name.Length;
-			System.IO.Stream bimstream = new System.IO.MemoryStream (marker.Data, len, marker.Data.Length - len, false);
-			FSpot.Bim.BimFile bim = new FSpot.Bim.BimFile (bimstream);
-					bim.Select (sink);
+			using (System.IO.Stream bimstream = new System.IO.MemoryStream (marker.Data, len, marker.Data.Length - len, false)) {
+				FSpot.Bim.BimFile bim = new FSpot.Bim.BimFile (bimstream);
+				bim.Select (sink);
+			}
 		}
 	}
 
@@ -361,14 +403,14 @@ public class JpegHeader : SemWeb.StatementSource {
 	
 	public void SetXmp (XmpFile xmp)
 	{
-		MemoryStream stream = new MemoryStream ();
-
-		XmpSignature.WriteName (stream);
-		xmp.Save (stream);
-
-		Marker xmp_marker = new Marker (XmpSignature.Id, stream.ToArray ());
-
-		Replace (XmpSignature, xmp_marker);
+		using (MemoryStream stream = new MemoryStream ()) {
+			
+			XmpSignature.WriteName (stream);
+			xmp.Save (stream);
+			
+			Marker xmp_marker = new Marker (XmpSignature.Id, stream.ToArray ());
+			Replace (XmpSignature, xmp_marker);
+		}
 	}
 	
 	public System.Collections.ArrayList Markers {
@@ -380,8 +422,8 @@ public class JpegHeader : SemWeb.StatementSource {
 	public void Save (System.IO.Stream stream)
 	{
 		foreach (Marker marker in marker_list) {
-			System.Console.WriteLine ("saving marker {0} {1}", marker.Type, 
-						   (marker.Data != null) ? marker.Data.Length .ToString (): "(null)");
+			//System.Console.WriteLine ("saving marker {0} {1}", marker.Type, 
+			//			   (marker.Data != null) ? marker.Data.Length .ToString (): "(null)");
 			marker.Save (stream);
 			if (marker.Type == JpegMarker.Sos)
 				stream.Write (ImageData, 0, ImageData.Length);
@@ -398,13 +440,6 @@ public class JpegHeader : SemWeb.StatementSource {
 		Load (stream, metadata_only);
 	}
 
-	public JpegHeader (string filename) 
-	{
-		System.Console.WriteLine ("opening {0}", filename);
-		System.IO.FileStream stream = new System.IO.FileStream (filename, System.IO.FileMode.Open);
-		Load (stream, false);
-	}
-
 	private void Load (System.IO.Stream stream, bool metadata_only) 
 	{
 		marker_list.Clear ();
@@ -429,8 +464,10 @@ public class JpegHeader : SemWeb.StatementSource {
 			if (marker.Type == JpegMarker.Sos) {
 				at_image = true;
 
-				if (metadata_only)
+				if (metadata_only) {
+					System.Console.WriteLine ("read = {0}", stream.Position);
 					return;
+				}
 			}
 		}
 
@@ -441,6 +478,109 @@ public class JpegHeader : SemWeb.StatementSource {
 			throw new System.Exception ("truncated image data or something");
 	}
 
+	static int [] StandardLuminanceQuantization = new int [] {
+	        16,  11,  12,  14,  12,  10,  16,  14,
+		13,  14,  18,  17,  16,  19,  24,  40,
+		26,  24,  22,  22,  24,  49,  35,  37,
+		29,  40,  58,  51,  61,  60,  57,  51,
+		56,  55,  64,  72,  92,  78,  64,  68,
+		87,  69,  55,  56,  80, 109,  81,  87,
+		95,  98, 103, 104, 103,  62,  77, 113,
+		121, 112, 100, 120,  92, 101, 103,  99
+	};
+
+	static int [] StandardChromninaceQuantization = new int [] {
+		17,  18,  18,  24,  21,  24,  47,  26,
+		26,  47,  99,  66,  56,  66,  99,  99,
+		99,  99,  99,  99,  99,  99,  99,  99,
+		99,  99,  99,  99,  99,  99,  99,  99,
+		99,  99,  99,  99,  99,  99,  99,  99,
+		99,  99,  99,  99,  99,  99,  99,  99,
+		99,  99,  99,  99,  99,  99,  99,  99,
+		99,  99,  99,  99,  99,  99,  99,  99
+	};
+	
+	/* 
+	 * GuessQuality is taken from the jpegdump utility
+	 * Copyright (c) 1992 Handmade Software, Inc.
+	 * by Allan N. Hessenflow licenced as GPL with the authors
+	 * permission.  Many Thanks.
+	 */
+	public int GuessQuality ()
+	{
+		Marker dqt = FindMarker (QuantizationSignature);
+		int quality = 0;
+		int position = 0;
+
+		while (position < dqt.Data.Length) {
+			int tableindex;
+			int [] table = null;
+			double cumsf = 0.0;
+			double cumsf2 = 0.0;
+			bool allones = true;
+			int row, col;
+			
+			tableindex = dqt.Data [position ++];
+
+			switch (tableindex & 0x0f) {
+			case 1:
+				table = StandardLuminanceQuantization;
+				break;
+			case 2:
+				table = StandardChromninaceQuantization;
+				break;
+			default:
+				table = null;
+				break;
+			}
+
+			for (row=0; row<8; row++) {
+				for (col=0; col<8; col++) {
+					uint val;
+					
+					if ((tableindex >> 4) > 0) {
+					        val = FSpot.BitConverter.ToUInt16 (dqt.Data, position, false);
+						position += 2;
+					} else
+						val = (uint) dqt.Data [position ++];
+
+					if (table != null) {
+						double x;
+
+						/* scaling factor in percent */
+						x = 100.0 * (double)val / (double)table [row*8+col];
+						cumsf += x;
+						cumsf2 += x * x;
+
+						/* separate check for all-ones table (Q 100) */
+						if (val != 1) 
+							allones = false;
+					}
+				}
+			}
+
+			if (table != null) {
+				double local_quality;
+				
+				cumsf /= 64.0;	/* mean scale factor */
+				cumsf2 /= 64.0;
+
+				//double variance;
+				//variance = cumsf2 - (cumsf * cumsf);
+
+				if (allones) /* special case for all-ones table */
+					local_quality = 100.0;
+				else if (cumsf <= 100.0)
+					local_quality = (200.0 - cumsf) / 2.0;
+				else
+					local_quality = 5000.0 / cumsf;
+				
+				quality = Math.Max (quality, (int)local_quality);
+			}
+		}
+		return quality;
+	}
+	
 	public byte [] ImageData {
 		get {
 			return image_data;
