@@ -16,43 +16,53 @@ using Gtk;
 using Gdk;
 using Mono.Posix;
 
-public class PhotoPopup {
-	public void Activate ()
-	{
-		Activate (null);
+public class PhotoPopup : Gtk.Menu {
+	Widget creator;
+
+	public Widget Creator {
+		get {
+			return creator;
+		}
 	}
 
-	public void Activate (Gdk.EventButton eb) 
+	public void Activate (Widget toplevel)
+	{
+		Activate (toplevel, null);
+	}
+
+	public void Activate (Widget toplevel, Gdk.EventButton eb) 
 	{
 		// FIXME this is a hack to handle the --view case for the time being.
+		creator = toplevel;
+
 		if (MainWindow.Toplevel == null)
 			return;
 
 		int count = MainWindow.Toplevel.SelectedIds ().Length;
 		
-		Gtk.Menu popup_menu = new Gtk.Menu ();
+		Gtk.Menu popup_menu = this;
 		bool have_selection = count > 0;
 		
 		GtkUtil.MakeMenuItem (popup_menu, Mono.Posix.Catalog.GetString ("Copy Photo Location"), 
-				      new EventHandler (MainWindow.Toplevel.HandleCopyLocation), have_selection);
+				      delegate { MainWindow.Toplevel.HandleCopyLocation (creator, null); }, have_selection);
 		
 		GtkUtil.MakeMenuSeparator (popup_menu);
 
 		GtkUtil.MakeMenuItem (popup_menu, "f-spot-rotate-270",
-				      new EventHandler (MainWindow.Toplevel.HandleRotate270Command), have_selection);
+				      delegate { MainWindow.Toplevel.HandleRotate270Command(creator, null); }, have_selection);
 		GtkUtil.MakeMenuItem (popup_menu, "f-spot-rotate-90", 
-				      new EventHandler (MainWindow.Toplevel.HandleRotate90Command), have_selection);
+				      delegate { MainWindow.Toplevel.HandleRotate90Command (creator, null); }, have_selection);
 
 		GtkUtil.MakeMenuSeparator (popup_menu);
 
 		OpenWithMenu owm = OpenWithMenu.AppendMenuTo (popup_menu, MainWindow.Toplevel.SelectedMimeTypes);
 		owm.IgnoreApp = "f-spot";
-		owm.ApplicationActivated += MainWindow.Toplevel.HandleOpenWith;
+		owm.ApplicationActivated += delegate (Gnome.Vfs.MimeApplication app) { MainWindow.Toplevel.HandleOpenWith (creator, app); };
 
 		GtkUtil.MakeMenuItem (popup_menu, Mono.Posix.Catalog.GetString ("Remove From Catalog"), 
-				      new EventHandler (MainWindow.Toplevel.HandleRemoveCommand), have_selection);
+				      delegate { MainWindow.Toplevel.HandleRemoveCommand (creator, null); }, have_selection);
 		GtkUtil.MakeMenuItem (popup_menu, Mono.Posix.Catalog.GetString ("Delete From Drive"),
-				      new EventHandler (MainWindow.Toplevel.HandleDeleteCommand), have_selection);
+				      delegate { MainWindow.Toplevel.HandleDeleteCommand (creator, null); }, have_selection);
 
 		GtkUtil.MakeMenuSeparator (popup_menu);
 		
@@ -61,7 +71,7 @@ public class PhotoPopup {
 		//
 		MenuItem attach_item = new MenuItem (Mono.Posix.Catalog.GetString ("Attach Tag"));
 		TagMenu attach_menu = new TagMenu (attach_item, MainWindow.Toplevel.Database.Tags);
-		attach_menu.NewTagHandler = MainWindow.Toplevel.HandleCreateTagAndAttach;
+		attach_menu.NewTagHandler += delegate { MainWindow.Toplevel.HandleCreateTagAndAttach (creator, null); };
 		attach_menu.TagSelected += MainWindow.Toplevel.HandleAttachTagMenuSelected;
 		attach_item.ShowAll ();
 		popup_menu.Append (attach_item);
@@ -82,5 +92,7 @@ public class PhotoPopup {
 			popup_menu.Popup (null, null, null, eb.Button, eb.Time);
 		else 
 			popup_menu.Popup (null, null, null, 0, Gtk.Global.CurrentEventTime);
-	}   
+	}
+
+	
 }
