@@ -288,15 +288,14 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 			return GetPathForVersionName (GetVersionName (version_id));
 	}
 
-	public string DefaultVersionPath {
-		get {
-			return GetVersionPath (DefaultVersionId);
-		}
+	public System.Uri VersionUri (uint version_id)
+	{
+		return UriList.PathToFileUri (GetVersionPath (version_id));
 	}
-
+	
 	public System.Uri DefaultVersionUri {
 		get {
-			return UriList.PathToFileUri (DefaultVersionPath);
+			return VersionUri (DefaultVersionId);
 		}
 	}
 
@@ -313,6 +312,14 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 		string path = GetVersionPath (version_id);
 		if (File.Exists (path))
 			File.Delete (path);
+
+		try {
+			string thumb_path = ThumbnailGenerator.ThumbnailPath (path);
+			File.Delete (thumb_path);
+		} catch (System.Exception e) {
+			//ignore an error here we don't really care.
+		}
+
 		PhotoStore.DeleteThumbnail (path);
 
 		version_names.Remove (version_id);
@@ -491,9 +498,9 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 
 	public void WriteMetadataToImage ()
 	{
-		string path = this.DefaultVersionPath;
+		string path = this.DefaultVersionUri.LocalPath;
 
-		FSpot.ImageFile img = FSpot.ImageFile.Create (path);
+		FSpot.ImageFile img = FSpot.ImageFile.Create (DefaultVersionUri);
 		if (img is FSpot.JpegFile) {
 			FSpot.JpegFile jimg = img as FSpot.JpegFile;
 			
@@ -517,7 +524,7 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 	public uint SaveVersion (Gdk.Pixbuf buffer, bool create_version)
 	{
 		uint version = DefaultVersionId;
-		ImageFile img = ImageFile.Create (DefaultVersionUri.LocalPath);
+		ImageFile img = ImageFile.Create (DefaultVersionUri);
 
 		// Always create a version if the source is not a jpeg for now.
 		create_version = create_version || !(img is FSpot.JpegFile);
@@ -594,11 +601,10 @@ public class PhotoStore : DbStore {
 	// Generates the thumbnail, returns the Pixbuf, and also stores it as a side effect
 	//
 
-	public static Pixbuf GenerateThumbnail (string path)
+	public static Pixbuf GenerateThumbnail (Uri uri)
 	{
-		string uri = UriList.PathToFileUri (path).ToString ();
 		Pixbuf thumbnail = null;
-		FSpot.ImageFile img = FSpot.ImageFile.Create (path);
+		FSpot.ImageFile img = FSpot.ImageFile.Create (uri);
 
 		if (img is FSpot.IThumbnailContainer) {
 			try {
@@ -610,10 +616,10 @@ public class PhotoStore : DbStore {
 
 		// Save embedded thumbnails in a silightly invalid way so that we know to regnerate them later.
 		if (thumbnail != null) 
-			PixbufUtils.SaveAtomic (thumbnail, Thumbnail.PathForUri (uri, ThumbnailSize.Large), 
+			PixbufUtils.SaveAtomic (thumbnail, FSpot.ThumbnailGenerator.ThumbnailPath (uri), 
 						"png", new string [] { null} , new string [] { null});
 		else 
-			thumbnail = FSpot.ThumbnailGenerator.Create (path);
+			thumbnail = FSpot.ThumbnailGenerator.Create (uri);
 		
 		return thumbnail;
 	}
@@ -724,7 +730,7 @@ public class PhotoStore : DbStore {
 		AddToCache (photo);
 		photo.Loaded = true;
 
-		thumbnail = GenerateThumbnail (newPath);		
+		thumbnail = GenerateThumbnail (UriList.PathToFileUri (newPath));		
 		EmitAdded (photo);
 
 		return photo;
@@ -948,8 +954,6 @@ public class PhotoStore : DbStore {
 		string directory_path = System.IO.Path.GetDirectoryName (path);
 		string filename = System.IO.Path.GetFileName (path);
 
-		directory_path = directory_path.Trim();
-		filename = filename.Trim();
 		command.CommandText = String.Format ("SELECT id,                                   " +
 				                     "       time,                                 " +
 						     "       description,                          " +
@@ -1227,7 +1231,7 @@ public class PhotoStore : DbStore {
 					"(SELECT DISTINCT photo_id FROM photo_tags) ");
 		
 		if (range != null) {
-			query_builder.Append (String.Format ("AND photos.time >= {0} AND photos.time < {1} ",
+			query_builder.Append (String.Format ("AND photos.time >= {0} AND photos.time <= {1} ",
 							     DbUtils.UnixTimeFromDateTime (range.Start), 
 							     DbUtils.UnixTimeFromDateTime (range.End)));
 		}
@@ -1275,7 +1279,7 @@ public class PhotoStore : DbStore {
 				      "     FROM photos                      ");
 		
 		if (range != null) {
-			query_builder.Append (String.Format ("WHERE photos.time >= {0} AND photos.time < {1} ",
+			query_builder.Append (String.Format ("WHERE photos.time >= {0} AND photos.time <= {1} ",
 							     DbUtils.UnixTimeFromDateTime (range.Start), 
 							     DbUtils.UnixTimeFromDateTime (range.End)));
 		}
