@@ -6,43 +6,18 @@ using System;
 using Mono.Unix;
 using FSpot.Xmp;
 
+namespace FSpot {
 public class PhotoView : EventBox {
 	FSpot.Delay description_delay; 
 
 	private bool has_selection = false;
-
-	public FSpot.PhotoImageView View {
-		get {
-			return photo_view;
-		}
-	}
-
-	new public FSpot.BrowsablePointer Item {
-		get {
-			return photo_view.Item;
-		}
-	}
-
-	private PhotoStore photo_store;
-
-	private FSpot.PhotoQuery query;
-	public FSpot.PhotoQuery Query {
-		get {
-			return query;
-		}
-
-		set {
-			query = value;
-		}
-	}
-	
-	public void Reload ()
-	{
-		photo_view.Reload ();
-	}
-
 	private FSpot.PhotoImageView photo_view;
+	private ScrolledWindow photo_view_scrolled;
+	private EventBox background;
+
 	private TagView tag_view;
+	private EventBox tag_view_box;
+	
 	private Button display_next_button, display_previous_button;
 	private Label count_label;
 	private Entry description_entry;
@@ -56,26 +31,11 @@ public class PhotoView : EventBox {
 
 	FSpot.AsyncPixbufLoader loader = new FSpot.AsyncPixbufLoader ();
 
-	public double Zoom {
-		get {
-			return photo_view.Zoom;
-		}
-
-		set {
-			photo_view.Zoom = value;
-		}
-	}
-	
-	public double NormalizedZoom {
-		get {
-			return photo_view.NormalizedZoom;
-		}
-
-		set {
-			photo_view.NormalizedZoom = value;
-		}
-	}
+	private OptionMenu constraints_option_menu;
+	private int selection_constraint_ratio_idx;
+	private uint restore_scrollbars_idle_id;
 
+	private System.Collections.Hashtable constraint_table = new System.Collections.Hashtable ();
 
 	// Public events.
 
@@ -102,23 +62,43 @@ public class PhotoView : EventBox {
 		}
 	}
 
-	private OptionMenu constraints_option_menu;
-	private int selection_constraint_ratio_idx;
-
 	private static SelectionConstraint [] constraints = {
-		new SelectionConstraint (Mono.Posix.Catalog.GetString ("No Constraint"), 0.0),
-		new SelectionConstraint (Mono.Posix.Catalog.GetString ("4 x 3 (Book)"), 4.0 / 3.0),
-		new SelectionConstraint (Mono.Posix.Catalog.GetString ("4 x 6 (Postcard)"), 6.0 / 4.0),
-		new SelectionConstraint (Mono.Posix.Catalog.GetString ("5 x 7 (L, 2L)"), 7.0 / 5.0),
-		new SelectionConstraint (Mono.Posix.Catalog.GetString ("8 x 10"), 10.0 / 8.0),
-		new SelectionConstraint (Mono.Posix.Catalog.GetString ("4 x 3 Portrait (Book)"), 3.0 / 4.0),
-		new SelectionConstraint (Mono.Posix.Catalog.GetString ("4 x 6 Portrait (Postcard)"), 4.0 / 6.0),
-		new SelectionConstraint (Mono.Posix.Catalog.GetString ("5 x 7 Portrait (L, 2L)"), 5.0 / 7.0),
-		new SelectionConstraint (Mono.Posix.Catalog.GetString ("8 x 10 Portrait"), 8.0 / 10.0),
-		new SelectionConstraint (Mono.Posix.Catalog.GetString ("Square"), 1.0)
+		new SelectionConstraint (Catalog.GetString ("No Constraint"), 0.0),
+		new SelectionConstraint (Catalog.GetString ("4 x 3 (Book)"), 4.0 / 3.0),
+		new SelectionConstraint (Catalog.GetString ("4 x 6 (Postcard)"), 6.0 / 4.0),
+		new SelectionConstraint (Catalog.GetString ("5 x 7 (L, 2L)"), 7.0 / 5.0),
+		new SelectionConstraint (Catalog.GetString ("8 x 10"), 10.0 / 8.0),
+		new SelectionConstraint (Catalog.GetString ("4 x 3 Portrait (Book)"), 3.0 / 4.0),
+		new SelectionConstraint (Catalog.GetString ("4 x 6 Portrait (Postcard)"), 4.0 / 6.0),
+		new SelectionConstraint (Catalog.GetString ("5 x 7 Portrait (L, 2L)"), 5.0 / 7.0),
+		new SelectionConstraint (Catalog.GetString ("8 x 10 Portrait"), 8.0 / 10.0),
+		new SelectionConstraint (Catalog.GetString ("Square"), 1.0)
 	};
 
-	private System.Collections.Hashtable constraint_table = new System.Collections.Hashtable ();
+	public FSpot.PhotoImageView View {
+		get { return photo_view; }
+	}
+
+	new public FSpot.BrowsablePointer Item {
+		get { return photo_view.Item; }
+	}
+
+
+	private IBrowsableCollection query;
+	public IBrowsableCollection Query {
+		get { return query; }
+		set { query = value; }
+	}
+
+	public double Zoom {
+		get { return photo_view.Zoom; }
+		set { photo_view.Zoom = value; }
+	}
+	
+	public double NormalizedZoom {
+		get { return photo_view.NormalizedZoom; }
+		set { photo_view.NormalizedZoom = value; }
+	}
 
 	private void HandleSelectionConstraintOptionMenuActivated (object sender, EventArgs args)
 	{
@@ -126,6 +106,11 @@ public class PhotoView : EventBox {
 		photo_view.SelectionXyRatio = constraints [selection_constraint_ratio_idx].XyRatio;
 	}
 
+	public void Reload ()
+	{
+		photo_view.Reload ();
+	}
+
 	private OptionMenu CreateConstraintsOptionMenu ()
 	{
 		Menu menu = new Menu ();
@@ -147,8 +132,10 @@ public class PhotoView : EventBox {
 		return constraints_option_menu;
 	}
 
-	private uint restore_scrollbars_idle_id;
-
+	private void ItemChanged (BrowsablePointer item, BrowsablePointerChangedArgs args)
+	{
+		
+	}
 
 	private void UpdateButtonSensitivity ()
 	{
@@ -156,19 +143,22 @@ public class PhotoView : EventBox {
 		bool prev = valid && Item.Index > 0;
 		bool next = valid && Item.Index < query.Count - 1;
 
+		if (valid) {
+			Gnome.Vfs.Uri vfs = new Gnome.Vfs.Uri (photo_view.Item.Current.DefaultVersionUri.ToString ());
+			valid = vfs.IsLocal;
+		}
+
 		display_previous_button.Sensitive = prev;
 		display_next_button.Sensitive = next;
 
-		if (valid) {
-			if (has_selection) {
-				tips.SetTip (crop_button, Catalog.GetString ("Crop photo to selected area"), "");
-				tips.SetTip (redeye_button, Catalog.GetString ("Remove redeye from selected area"), "");
-			} else {
-				tips.SetTip (crop_button, Catalog.GetString ("Select an area to crop"), null);
-				tips.SetTip (redeye_button, Catalog.GetString ("Select an area to remove redeye"), null);
-			}
+		if (valid && has_selection) {
+			tips.SetTip (crop_button, Catalog.GetString ("Crop photo to selected area"), "");
+			tips.SetTip (redeye_button, Catalog.GetString ("Remove redeye from selected area"), "");
+		} else {
+			tips.SetTip (crop_button, Catalog.GetString ("Select an area to crop"), null);
+			tips.SetTip (redeye_button, Catalog.GetString ("Select an area to remove redeye"), null);
 		}
-
+		
 		crop_button.Sensitive = valid;
 		redeye_button.Sensitive = valid;
 		color_button.Sensitive = valid;
@@ -181,7 +171,7 @@ public class PhotoView : EventBox {
 		if (query == null)
 			count_label.Text = "";
 		else {
-			if (Query.Photos.Length == 0)
+			if (query.Count == 0)
 				count_label.Text = String.Format ("{0} of {1}", 0, 0);
 			else 
 				count_label.Text = String.Format ("{0} of {1}", Item.Index + 1, Query.Count);
@@ -194,9 +184,10 @@ public class PhotoView : EventBox {
 		if (Item.IsValid) {
 			if (description_entry.Sensitive == false)
 				description_entry.Sensitive = true;
-			
-			if (description_entry.Text != Item.Current.Description) {
-				description_entry.Text = Item.Current.Description;
+
+			string desc = Item.Current.Description;
+			if (description_entry.Text != desc) {
+				description_entry.Text = desc == null ? "" : desc;
 			}
 		} else {
 			description_entry.Sensitive = false;
@@ -271,21 +262,33 @@ public class PhotoView : EventBox {
 	{
 		ProcessImage (false);
 	}
-	
-	private void ShowError (System.Exception e, FSpot.IBrowsableItem photo)
+
+	private void ShowError (System.Exception e, Photo photo)
 	{
-		Dialog md = new FSpot.EditExceptionDialog ((Gtk.Window)this.Toplevel, e, photo); 
+		string msg = Catalog.GetString ("Error editing photo");
+		string desc = String.Format (Catalog.GetString ("Received exception \"{0}\". Unable to save photo {1}"),
+					     e.Message, photo.Name);
+		
+		HigMessageDialog md = new HigMessageDialog ((Gtk.Window)this.Toplevel, DialogFlags.DestroyWithParent, 
+							    Gtk.MessageType.Error, ButtonsType.Ok, 
+							    msg,
+							    desc);
 		md.Run ();
 		md.Destroy ();
 	}
 
 	private void HandleSepiaButtonClicked (object sender, EventArgs args)
 	{
+		PhotoQuery pq = query as PhotoQuery;
+
+		if (pq == null)
+			return;
+
 		try {
 			FSpot.SepiaTone sepia = new FSpot.SepiaTone ((Photo)View.Item.Current);
 			sepia.Pixbuf = View.CompletePixbuf ();
 			sepia.Adjust ();
-			query.Commit (Item.Index);
+			pq.Commit (Item.Index);
 		} catch (System.Exception e) {
 			ShowError (e, (Photo)View.Item.Current); 
 		}
@@ -293,11 +296,16 @@ public class PhotoView : EventBox {
 
 	private void HandleDesaturateButtonClicked (object sender, EventArgs args)
 	{
+		PhotoQuery pq = query as PhotoQuery;
+
+		if (pq == null)
+			return;
+
 		try {
 			FSpot.Desaturate desaturate = new FSpot.Desaturate ((Photo) View.Item.Current);
 			desaturate.Pixbuf = View.CompletePixbuf ();
 			desaturate.Adjust ();
-			query.Commit (Item.Index);
+			pq.Commit (Item.Index);
 		} catch (System.Exception e) {
 			ShowError (e, (Photo)View.Item.Current);
 		}
@@ -309,8 +317,8 @@ public class PhotoView : EventBox {
 	{
 		int x, y, width, height;
 		if (! photo_view.GetSelection (out x, out y, out width, out height)) {
-			string msg = Mono.Posix.Catalog.GetString ("No selection available");
-			string desc = Mono.Posix.Catalog.GetString ("This tool requires an active selection. Please select a region of the photo and try the operation again");
+			string msg = Catalog.GetString ("No selection available");
+			string desc = Catalog.GetString ("This tool requires an active selection. Please select a region of the photo and try the operation again");
 			
 			HigMessageDialog md = new HigMessageDialog ((Gtk.Window)this.Toplevel, DialogFlags.DestroyWithParent, 
 								    Gtk.MessageType.Error, ButtonsType.Ok, 
@@ -321,21 +329,15 @@ public class PhotoView : EventBox {
 			md.Destroy ();
 			return;
 		}		
-		
+
 		Photo photo = (Photo)Item.Current;
 		try {
-			if (! System.IO.File.Exists (photo.DefaultVersionPath)) {
-				throw new System.IO.FileNotFoundException ();
-			}
-
 			Pixbuf original_pixbuf = photo_view.CompletePixbuf ();
-
 			if (original_pixbuf == null) {
 				return;
 			}
 			
 			Pixbuf edited;
-
 			if (redeye) {
 				Gdk.Rectangle area = new Gdk.Rectangle (x, y, width, height);
 				edited = PixbufUtils.RemoveRedeye (original_pixbuf, 
@@ -348,24 +350,22 @@ public class PhotoView : EventBox {
 				original_pixbuf.CopyArea (x, y, width, height, edited, 0, 0);
 			}
 			
-			
-			// FIXME the fact that the selection doesn't go away is a bug in ImageView, it should
-			// be fixed there.
-			photo_view.UnsetSelection ();
-			
 			bool create_version = photo.DefaultVersionId == Photo.OriginalVersionId;
 			photo.SaveVersion (edited, create_version);
+			((PhotoQuery)query).Commit (Item.Index);
+
+			// FIXME the fact that the selection doesn't go away is a bug in ImageView, it should
+			// be fixed there.
 			photo_view.Pixbuf = edited;
-			query.Commit (Item.Index);
-			query.MarkChanged (Item.Index);
+			photo_view.UnsetSelection ();
+
+			photo_view.Fit = true;
+			
+			if (PhotoChanged != null)
+				PhotoChanged (this);
 		} catch (System.Exception e) {
 			ShowError (e, photo);
 		}
-		
-		photo_view.Fit = true;
-
-		if (PhotoChanged != null)
-			PhotoChanged (this);
 	}
 
 	private void HandleColorButtonClicked (object sender, EventArgs args) 
@@ -378,30 +378,29 @@ public class PhotoView : EventBox {
 	{
 		if (description_delay.IsPending) {
 			description_delay.Stop ();
-			Query.Commit (description_photo);
+			((PhotoQuery)query).Commit (description_photo);
 		}
 		return true;
 	}
 
-	private void HandleDescriptionChanged (object sender, EventArgs args) {
+	private void HandleDescriptionChanged (object sender, EventArgs args) 
+	{
 		if (!Item.IsValid)
 			return;
-
+		
 		((Photo)Item.Current).Description = description_entry.Text;
-
+		
 		if (description_delay.IsPending)
 			if (description_photo == Item.Index)
 				description_delay.Stop ();
 			else
 				CommitPendingChanges ();
-
+		
 		tips.SetTip (description_entry, description_entry.Text, "This is a tip");
 		description_photo = Item.Index;
 		description_delay.Start ();
 	}
 
-	
-
 	// Constructor.
 
 	private class ToolbarButton : Button {
@@ -415,11 +414,13 @@ public class PhotoView : EventBox {
 
 	private void HandlePhotoChanged (FSpot.PhotoImageView view)
 	{
-		CommitPendingChanges ();
+		if (query is PhotoQuery) {
+			CommitPendingChanges ();
+		}
+		
+		tag_view.Current = Item.Current;
 		Update ();
 
-		tag_view.Current = (Photo)(Item.Current);
-
 		if (this.PhotoChanged != null)
 			PhotoChanged (this);
 	}
@@ -441,11 +442,10 @@ public class PhotoView : EventBox {
 
 	Gtk.Tooltips tips = new Gtk.Tooltips ();
 
-	public PhotoView (FSpot.PhotoQuery query, PhotoStore photo_store)
+	public PhotoView (IBrowsableCollection query)
 		: base ()
 	{
 		this.query = query;
-		this.photo_store = photo_store;
 
 		description_delay = new FSpot.Delay (1000, new GLib.IdleHandler (CommitPendingChanges));
 		this.Destroyed += HandleDestroy;
@@ -454,11 +454,12 @@ public class PhotoView : EventBox {
 		Box vbox = new VBox (false, 6);
 		Add (vbox);
 
-		EventBox eventbox = new EventBox ();
+	        background = new EventBox ();
 		Frame frame = new Frame ();
-		eventbox.Add (frame);
+		background.Add (frame);
+
 		frame.ShadowType = ShadowType.In;
-		vbox.PackStart (eventbox, true, true, 0);
+		vbox.PackStart (background, true, true, 0);
 		
 		Box inner_vbox = new VBox (false , 2);
 
@@ -468,12 +469,7 @@ public class PhotoView : EventBox {
 		photo_view.PhotoChanged += HandlePhotoChanged;
 		photo_view.SelectionChanged += HandleSelectionChanged;
 
-		ScrolledWindow photo_view_scrolled = new ScrolledWindow (null, null);
-
-
-		FSpot.Global.ModifyColors (photo_view);
-		FSpot.Global.ModifyColors (eventbox);
-		FSpot.Global.ModifyColors (photo_view_scrolled);
+		photo_view_scrolled = new ScrolledWindow (null, null);
 
 		photo_view_scrolled.SetPolicy (PolicyType.Automatic, PolicyType.Automatic);
 		photo_view_scrolled.ShadowType = ShadowType.None;
@@ -485,11 +481,11 @@ public class PhotoView : EventBox {
 		HBox inner_hbox = new HBox (false, 2);
 		//inner_hbox.BorderWidth = 6;
 
-		EventBox tag_view_box = new EventBox ();
+		tag_view_box = new EventBox ();
 		tag_view = new TagView (tag_view_box);
 		tag_view_box.Add (tag_view);
 		inner_hbox.PackStart (tag_view_box, false, true, 0);
-		FSpot.Global.ModifyColors (tag_view_box);
+		SetColors ();
 
 		Label comment = new Label (Catalog.GetString ("Comment:"));
 		inner_hbox.PackStart (comment, false, false, 0);
@@ -556,13 +552,25 @@ public class PhotoView : EventBox {
 		UpdateButtonSensitivity ();
 
 		vbox.ShowAll ();
-		tips.SetTip (color_button, Mono.Posix.Catalog.GetString ("Adjust the photo colors"), "");
-		tips.SetTip (constraints_option_menu, Mono.Posix.Catalog.GetString ("Constrain the aspect ratio of the selection"), "");
-		tips.SetTip (display_next_button, Mono.Posix.Catalog.GetString ("Next photo"), "");
-		tips.SetTip (display_previous_button, Mono.Posix.Catalog.GetString ("Previous photo"), "");
-		tips.SetTip (desaturate_button, Mono.Posix.Catalog.GetString ("Convert the photo to black and white"), "");
-		tips.SetTip (sepia_button, Mono.Posix.Catalog.GetString ("Convert the photo to sepia tones"), "");
+		tips.SetTip (color_button, Catalog.GetString ("Adjust the photo colors"), "");
+		tips.SetTip (constraints_option_menu, Catalog.GetString ("Constrain the aspect ratio of the selection"), "");
+		tips.SetTip (display_next_button, Catalog.GetString ("Next photo"), "");
+		tips.SetTip (display_previous_button, Catalog.GetString ("Previous photo"), "");
+		tips.SetTip (desaturate_button, Catalog.GetString ("Convert the photo to black and white"), "");
+		tips.SetTip (sepia_button, Catalog.GetString ("Convert the photo to sepia tones"), "");
+	}
+	
+	private void SetColors ()
+	{
+		FSpot.Global.ModifyColors (tag_view_box);
+		FSpot.Global.ModifyColors (photo_view);
+		FSpot.Global.ModifyColors (background);
+		FSpot.Global.ModifyColors (photo_view_scrolled);
 	}
 
+	protected override void OnStyleSet (Style previous)
+	{
+		SetColors ();
+	}
+}
 }
-
