@@ -12,6 +12,9 @@ using System.IO;
 8888      8888    8888  8888    88          8888888888  8888888888          88
 88          88      88  88
 88          88  888888  888888
+
+t-l     t-r     b-r     b-l     l-t         r-t         r-b             l-b
+
 **/
 
 public enum PixbufOrientation {
@@ -98,7 +101,8 @@ class PixbufUtils {
 	// FIXME: These should be in GTK#.  When my patch is committed, these LoadFrom* methods will
 	// go away.
 
-	public class AspectLoader : Gdk.PixbufLoader {
+	public class AspectLoader {
+		Gdk.PixbufLoader loader = new Gdk.PixbufLoader ();
 		int max_width;
 		int max_height;
 		PixbufOrientation orientation;
@@ -107,7 +111,7 @@ class PixbufUtils {
 		{
 			this.max_height = max_height;
 			this.max_width = max_width;
-			SizePrepared += HandleSizePrepared;
+			loader.SizePrepared += HandleSizePrepared;
 		}
 
 		private void HandleSizePrepared (object obj, SizePreparedArgs args)
@@ -133,38 +137,41 @@ class PixbufUtils {
 			int scale_height = (int)(scale * args.Height);
 
 			if (scale < 1.0)
-				SetSize (scale_width, scale_height);
+				loader.SetSize (scale_width, scale_height);
 		}
 
+		public Pixbuf Load (System.IO.Stream stream, PixbufOrientation orientation)
+		{
+			int count;
+			byte [] data = new byte [8192];
+			while (((count = stream.Read (data, 0, data.Length)) > 0) && loader.Write (data, (ulong)count))
+				;
+			
+			loader.Close ();
+			Pixbuf orig = loader.Pixbuf;
+			Gdk.Pixbuf rotated = TransformOrientation (orig, orientation, true);
+			
+			if (orig != rotated) {
+				CopyThumbnailOptions (orig, rotated);
+				orig.Dispose ();
+			}
+			loader.Dispose ();
+			return rotated;
+		}
+		
 		public Pixbuf LoadFromFile (string path)
 		{
 			FileStream fs = null;
-
+			
 			try {
 				orientation = GetOrientation (path);
-				fs = File.OpenRead (path);
-				int count;
-				
-				byte [] data = new byte [8192];
-				while (((count = fs.Read (data, 0, 8192)) > 0) && this.Write (data, (ulong)count))
-					;
-				
-				this.Close ();
-				Gdk.Pixbuf rotated = TransformOrientation (this.Pixbuf, orientation, true);
-				
-				if (this.Pixbuf != rotated) {
-					CopyThumbnailOptions (this.Pixbuf, rotated);
-					this.Pixbuf.Dispose ();
+				using (fs = File.OpenRead (path)) {
+					return Load (fs, orientation);
 				}
-				return rotated;
 			} catch (Exception e) {
-				System.Console.WriteLine ("Error loading image {0}", path);
+				System.Console.WriteLine ("Error loading photo {0}", path);
 				return null;
-			} finally {
-				if (fs != null)
-					fs.Close ();
-			}
-
+			} 
 		}
 	}
 
@@ -222,7 +229,9 @@ class PixbufUtils {
 			loader.Write (buffer, (ulong) n);
 		
 		loader.Close ();
+		
 		return loader.Pixbuf;
+		
 	}
 	
 
@@ -259,7 +268,8 @@ class PixbufUtils {
 	}
 
 	[DllImport("libgdk_pixbuf-2.0-0.dll")]
-	static extern bool gdk_pixbuf_save_to_bufferv (IntPtr raw, out IntPtr data, out uint length, string type, 
+	static extern bool gdk_pixbuf_save_to_bufferv (IntPtr raw, out IntPtr data, out IntPtr length, 
+						       string type, 
 						       string [] keys, string [] values, out IntPtr error);
 
 					
@@ -267,32 +277,40 @@ class PixbufUtils {
 	{
 		IntPtr error = IntPtr.Zero;
 		IntPtr data;
-		uint length;
-		string [] terminated_options = null;
-		string [] terminated_values = null;
+		IntPtr length;
+		string [] terminated_options = options;
+		string [] terminated_values = values;
 
-		if (options != null) {
+		if (options != null && options [ options.Length - 1 ] != null) {
 			terminated_options = new string [options.Length + 1];
 			Array.Copy (options, terminated_options, options.Length);
 		}
 
-		if (options != null) {
+		if (values != null && values [ values.Length - 1] != null) {
 			terminated_values = new string [values.Length + 1];
 			Array.Copy (values, terminated_values, values.Length);
 		}
+		
+		if (terminated_options != null)
+			System.Console.WriteLine ("options = {0}, values = {1}", terminated_options [0], terminated_values [0]);
+		else
+			Console.WriteLine ("no options");
 
-		gdk_pixbuf_save_to_bufferv (pixbuf.Handle, 
-					    out data, 
-					    out length, 
-					    type,
-					    terminated_options,
-					    terminated_values,
-					    out error);
-
+		bool success = gdk_pixbuf_save_to_bufferv (pixbuf.Handle, 
+							     out data, 
+							     out length, 
+							     type,
+							     terminated_options,
+							     terminated_values,
+							     out error);
+		
 		if (error != IntPtr.Zero) 
 			throw new GLib.GException (error);
 
-		byte [] content = new byte [length];
+		if (!success)
+			throw new ApplicationException ("Unknown error while saving file");
+
+		byte [] content = new byte [(int)length];
 		Marshal.Copy (data, content, 0, (int)length);
 
 		return content;
@@ -387,8 +405,6 @@ class PixbufUtils {
 		Pixbuf scaled = new Pixbuf (Colorspace.Rgb, false, 8, width, height);
 		scaled.Fill (0x000000); 
 
-		//System.Console.WriteLine ("pos= {0}", pos.ToString ());
-
 		orig.Composite (scaled, pos.X, pos.Y, 
 				pos.Width, pos.Height,
 				pos.X, pos.Y, scale, scale,
@@ -539,8 +555,11 @@ class PixbufUtils {
 			throw new Exception ("Invalid Dimensions");
 
 		//Cms.Profile eos10d = new Cms.Profile ("/home/lewing/ICCProfiles/EOS-10D-True-Color-Non-Linear.icm");
-		Cms.Profile srgb = Cms.Profile.CreateSRgb ();
-		Cms.Profile bchsw = Cms.Profile.CreateAbstract (10, brightness, contrast,
+		Cms.Profile srgb = Cms.Profile.CreateStandardRgb ();
+
+		Cms.Profile bchsw = Cms.Profile.CreateAbstract (256, 
+								0.0, 
+								brightness, contrast,
 								hue, saturation, src_color, 
 								dest_color);
 
@@ -755,23 +774,6 @@ class PixbufUtils {
 	}
 	// Bindings from libf.
 
-	[DllImport ("libfspot")]
-	static extern IntPtr f_pixbuf_copy_rotate_90 (IntPtr src, bool counter_clockwise);
-
-	public static Pixbuf Rotate90 (Pixbuf src, bool counter_clockwise)
-	{
-		return new Pixbuf (f_pixbuf_copy_rotate_90 (src.Handle, counter_clockwise));
-	}
-
-	[DllImport ("libfspot")]
-	static extern IntPtr f_pixbuf_copy_mirror (IntPtr src, bool mirror, bool flip);
-
-	// FIXME not very readable
-	public static Pixbuf Mirror (Pixbuf src, bool mirror, bool flip)
-	{
-		return new Pixbuf (f_pixbuf_copy_mirror (src.Handle, mirror, flip));
-	}
-
 	[DllImport ("libfspot")]
 	static extern IntPtr f_pixbuf_copy_apply_brightness_and_contrast (IntPtr src, float brightness, float contrast);
 
