@@ -1,4 +1,4 @@
-
+using System.Net;
 using System;
 using Mono.Posix;
 
@@ -70,12 +70,23 @@ namespace FSpot
 		public const string EXPORT_GALLERY_SIZE = "/apps/f-spot/export/gallery/size";
 		public const string EXPORT_GALLERY_BROWSER = "/apps/f-spot/export/gallery/browser";
 		public const string EXPORT_GALLERY_META = "/apps/f-spot/export/gallery/meta";
+		public const string EXPORT_GALLERY_ROTATE = "/apps/f-spot/export/gallery/rotate";
+
+		public const string EXPORT_PICASAWEB_SCALE = "/apps/f-spot/export/picasaweb/scale";
+		public const string EXPORT_PICASAWEB_SIZE = "/apps/f-spot/export/picasaweb/size";
+		public const string EXPORT_PICASAWEB_ROTATE = "/apps/f-spot/export/picasaweb/rotate";
+		public const string EXPORT_PICASAWEB_BROWSER = "/apps/f-spot/export/picasaweb/browser";
+
+ 		public const string EXPORT_EMAIL_SIZE = "/apps/f-spot/export/email/size";
+ 		public const string EXPORT_EMAIL_ROTATE = "/apps/f-spot/export/email/auto_rotate";
+ 		public const string EXPORT_EMAIL_DELETE_TIMEOUT_SEC = "/apps/f-spot/export/email/delete_timeout_seconds";
 
 		public const string SCREENSAVER_TAG = "/apps/f-spot/screensaver/tag_id";
 
 		public const string METADATA_EMBED_IN_IMAGE = "/apps/f-spot/metadata/embed_in_image";
 
 		public const string GNOME_SCREENSAVER_THEME = "/apps/gnome-screensaver/themes";
+		public const string GNOME_SCREENSAVER_MODE = "/apps/gnome-screensaver/mode";
 
 		static GConf.Client client;
 		static GConf.NotifyEventHandler changed_handler;
@@ -118,17 +129,42 @@ namespace FSpot
 			case SHOW_TIMELINE:
 			case SHOW_TAGS:
 			case SHOW_DATES:
+			case EXPORT_PICASAWEB_SCALE:
+			case EXPORT_PICASAWEB_ROTATE:
+			case EXPORT_PICASAWEB_BROWSER:
 				return true;
+			
+			case EXPORT_PICASAWEB_SIZE:
+				return 800;
 		
 			case SIDEBAR_POSITION:
 			case ZOOM:
 				return null;
 			case SCREENSAVER_TAG:
 				return 1;
+				
+			case EXPORT_EMAIL_SIZE:
+				return 3;	// medium size 640px
+			case EXPORT_EMAIL_ROTATE:
+				return true;	// auto rotate original photos
+			case EXPORT_EMAIL_DELETE_TIMEOUT_SEC:
+				return 30;	// delete temporary email pictures after 30 seconds
 			}
 
 			return null;
 		}
+		
+		public static object Get (string key, object default_val)
+		{
+			try {
+				return Client.Get (key);
+			} catch (GConf.NoSuchKeyException) {
+				if (default_val != null)
+					Client.Set (key, default_val);
+				
+				return default_val;
+			}
+		}
 
 		public static object Get (string key)
 		{
@@ -157,5 +193,49 @@ namespace FSpot
 				SettingChanged (sender, args);
 			}
 		}
+
+		private static string ProxyBase =     "/system/http_proxy";
+		private static string UseProxyKey =   ProxyBase + "/use_http_proxy";
+		private static string HostKey =       ProxyBase + "/host";
+		private static string PortKey =       ProxyBase + "/port";
+		private static string UserKey =       ProxyBase + "/authentication_user";
+		private static string PaswordKey =    ProxyBase + "/authentication_password";
+		private static string BypassListKey = ProxyBase + "/ignore_hosts";
+		
+		public static WebProxy GetProxy () 
+		{
+			WebProxy proxy = null;
+			
+			if ((bool) Preferences.Get (UseProxyKey, false))
+				return null;
+
+			try {
+				string host;
+				int    port;
+				
+				host = (string) Preferences.Get (HostKey, null);
+				port = (int) Preferences.Get (PortKey, 0);
+				
+				string uri = "http://" + host + ":" + port.ToString();
+				proxy = new WebProxy(uri);
+
+				string [] bypass_list = (string []) Preferences.Get (BypassListKey, null);
+				if (bypass_list != null) {
+					for (int i = 0; i < bypass_list.Length; i++) {
+						bypass_list [i] = "http://" + bypass_list [i];
+					}
+					proxy.BypassList = bypass_list;
+				}
+
+				string username = (string) Preferences.Get (UserKey, "");
+				string password = (string) Preferences.Get (PaswordKey, "");
+
+				proxy.Credentials = new NetworkCredential (username, password);
+			} catch (Exception e) {
+				proxy = null;
+			}
+
+			return proxy;
+		}
 	}
 }
