@@ -20,20 +20,28 @@ namespace FSpot.Raf {
 	}
 	
 	public class RafFile : ImageFile, SemWeb.StatementSource {
+
+                // false seems a safe default
+                public bool Distinct {
+                        get { return false; }
+                }
+
+		public RafFile (System.Uri uri) : base (uri)
+		{
+		}
+
 		public RafFile (string path) : base (path)
 		{
-			
 		}
 
 		public override System.IO.Stream PixbufStream ()
 		{
-			System.IO.MemoryStream stream = null; 
 			byte [] data = GetEmbeddedJpeg ();
 			
 			if (data != null)
-				stream = new System.IO.MemoryStream (data);
-				
-			return stream;
+				return new System.IO.MemoryStream (data);
+			else
+				return DCRawFile.RawPixbufStream (uri);
 		}
  
 		public override Gdk.Pixbuf Load ()
@@ -71,20 +79,20 @@ namespace FSpot.Raf {
 
 		private byte [] GetEmbeddedJpeg ()
 		{
-			using (System.IO.Stream stream = System.IO.File.OpenRead (this.path)) {
+			using (System.IO.Stream stream = Open ()) {
 				stream.Position = 0x54;
 				byte [] data = new byte [24];
 				stream.Read (data, 0, data.Length);
 				uint jpeg_offset = BitConverter.ToUInt32 (data, 0, false);
 				uint jpeg_length = BitConverter.ToUInt32 (data, 4, false);
 
-				uint wb_offset = BitConverter.ToUInt32 (data, 8, false);
-				uint wb_length = BitConverter.ToUInt32 (data, 12, false);
+				// FIXME implement wb parsing
+				//uint wb_offset = BitConverter.ToUInt32 (data, 8, false);
+				//uint wb_length = BitConverter.ToUInt32 (data, 12, false);
 				
-
-				uint raw_offset = BitConverter.ToUInt32 (data, 16, false);
-				uint raw_length = BitConverter.ToUInt32 (data, 20, false);
-
+				// FIXME implement decoding
+				//uint raw_offset = BitConverter.ToUInt32 (data, 16, false);
+				//uint raw_length = BitConverter.ToUInt32 (data, 20, false);
 
 				byte [] image = new byte [jpeg_length];
 				stream.Position = jpeg_offset;
