@@ -153,24 +153,49 @@ public class TagSelectionWidget : TreeView {
 		}
 	}
 
-	public Tag [] TagHighlight () {
-		TreeModel model;
+	public void ScrollTo (Tag tag)
+	{
 		TreeIter iter;
+		if (! TreeIterForTag (tag, out iter))
+			return;
 
-		TreePath [] rows = Selection.GetSelectedRows(out model);
+		TreePath path = Model.GetPath (iter);
 
-		Tag [] tags = new Tag [rows.Length];
-		int i = 0;
+		ScrollToCell (path, null, false, 0, 0);
+	}
 
-		foreach (TreePath path in rows) {
-			GLib.Value value = new GLib.Value ();
-			Model.GetIter (out iter, path);
-			Model.GetValue (iter, IdColumn, ref value);
-			uint tag_id = (uint) value;
-			tags[i] = tag_store.Get (tag_id) as Tag;
-			i++;
+	public Tag [] TagHighlight {
+		get {
+			TreeModel model;
+			TreeIter iter;
+
+			TreePath [] rows = Selection.GetSelectedRows(out model);
+
+			Tag [] tags = new Tag [rows.Length];
+			int i = 0;
+
+			foreach (TreePath path in rows) {
+				GLib.Value value = new GLib.Value ();
+				Model.GetIter (out iter, path);
+				Model.GetValue (iter, IdColumn, ref value);
+				uint tag_id = (uint) value;
+				tags[i] = tag_store.Get (tag_id) as Tag;
+				i++;
+			}
+			return tags;
+		}
+
+		set {
+			if (value == null)
+				return;
+
+			Selection.UnselectAll ();
+
+			TreeIter iter;
+			foreach (Tag tag in value)
+				if (TreeIterForTag (tag, out iter))
+					Selection.SelectIter (iter);
 		}
-		return tags;
 	}
 
 	public void Update ()
@@ -377,17 +402,20 @@ public class TagSelectionWidget : TreeView {
 			//I have no desire to figure out a more performant sort over this...
 			GLib.Value value = new GLib.Value ();
 			store.GetValue(iter, IdColumn, ref value);
-			compare = (Tag)tag_store.Get ((uint) value);
+			compare = (Tag) tag_store.Get ((uint) value);
 
 			if (compare.CompareTo (tag) > 0) {
 				iter = store.InsertNodeBefore (iter);
 				store.SetValue (iter, IdColumn, tag.Id);
 				store.SetValue (iter, NameColumn, tag.Name);
+				
+				if (!is_root)
+					ExpandRow (Model.GetPath (parent), false);
 				return iter;
 			}
 			valid = store.IterNext(ref iter);
 		}
-
+		
 		if (is_root) 
 			iter = store.AppendNode (); 
 		else {
@@ -398,7 +426,7 @@ public class TagSelectionWidget : TreeView {
 		store.SetValue (iter, IdColumn, tag.Id);
 		store.SetValue (iter, NameColumn, tag.Name);
 		return iter;
-	}	
+	}
 
 	private void HandleTagsRemoved (object sender, DbItemEventArgs args)
 	{
@@ -545,6 +573,16 @@ public class TagSelectionWidget : TreeView {
 		
 		SetCursor (rows[0], name_column, true);
 	}
+
+	// Override the default ButtonPressEvent because it is stupid and
+	// returns true, meaning if you want to have another handler for the ButtonPress
+	// event you have to Glib.ConnectBefore.  Plus you usually want the default one to
+	// run first so it will select/unselect anything necessary before popping up a menu, say.
+	protected override bool OnButtonPressEvent (Gdk.EventButton button)
+	{
+		base.OnButtonPressEvent (button);
+		return false;
+	}
 	
 	public void HandleTagNameEdited (object sender, EditedArgs args)
 	{
