@@ -1,5 +1,7 @@
+//#define DEBUG_LOADER
 using FSpot;
 using SemWeb;
+using System;
 
 namespace FSpot.Tiff {
 
@@ -184,7 +186,7 @@ namespace FSpot.Tiff {
 		SensingMethod			= 0xa217,
 		FileSource			= 0xa300,
 		SceneType			= 0xa301,
-		NewCFAPattern		        = 0xa302,
+		ExifCFAPattern		        = 0xa302,
 		CustomRendered  		= 0xa401,
 		ExposureMode			= 0xa402,
 		WhiteBalance			= 0xa403,
@@ -200,6 +202,7 @@ namespace FSpot.Tiff {
 		ImageUniqueId   		= 0xa420,
 
 		// The Following IDs are not described the EXIF spec
+		Gamma                           = 0xa500,
 
 		// The XMP spec declares that XMP data should live 0x2bc when
 		// embedded in tiff images.
@@ -258,6 +261,12 @@ namespace FSpot.Tiff {
 		public int Numerator;
 		public int Denominator;
 		
+		public SRational (byte [] raw_data, int offset, bool little)
+		{
+			Numerator = BitConverter.ToInt32 (raw_data, offset, little);
+			Denominator = BitConverter.ToInt32 (raw_data, offset, little);
+		}
+
 		public SRational (int numerator, int denominator)
 		{
 			Numerator = numerator;
@@ -275,10 +284,14 @@ namespace FSpot.Tiff {
 
 		public override string ToString ()
 		{
-			if (Denominator == 1)
-				return Numerator.ToString ();
+			if (Numerator == 0)
+				return "0";
+			else if (Numerator % Denominator == 0)
+				return String.Format ("{0}", Numerator / Denominator);
+			else if (Denominator % Numerator == 0)
+				return String.Format ("1/{0}", Denominator / Numerator);
 
-			return System.String.Format ("{0}/{1}", Numerator, Denominator);
+			return String.Format ("{0}/{1}", Numerator, Denominator);
 		}
 		
 		public double Value {
@@ -297,13 +310,32 @@ namespace FSpot.Tiff {
 			Numerator = numerator;
 			Denominator = denominator;
 		}
+		
+		public Rational (string value)
+		{
+			string [] vals = value.Split ('/');
+			if (vals.Length == 2) {
+				this.Numerator = UInt32.Parse (vals [0]);
+				this.Denominator = UInt32.Parse (vals [1]);
+				return;
+			} if (vals.Length == 1) {
+				double tmp = Double.Parse (value);
+				this.Numerator = (uint) (tmp * 100000);
+				this.Denominator = 100000;
+			} else
+				throw new System.Exception ("unable to parse rational value");
+		}
 
 		public override string ToString ()
 		{
-			if (Denominator == 1)
-				return Numerator.ToString ();
-			
-			return System.String.Format ("{0}/{1}", Numerator, Denominator);
+			if (Numerator == 0)
+				return "0";
+			else if (Numerator % Denominator == 0)
+				return String.Format ("{0}", Numerator / Denominator);
+			else if (Denominator % Numerator == 0)
+				return String.Format ("1/{0}", Denominator / Numerator);
+
+			return String.Format ("{0}/{1}", Numerator, Denominator);
 		}
 		
 		public double Value {
@@ -317,10 +349,23 @@ namespace FSpot.Tiff {
 		string Charset;
 		public string Value;
 
+		public UserComment (string value)
+		{
+			Charset = null;
+			Value = value;
+		}
+
 		public UserComment (byte [] raw_data, bool little)
 		{
+			if (raw_data.Length == 8 || raw_data.Length == 0) { 
+				Charset = null;
+				Value = "";
+				return;
+			} else if (raw_data.Length < 8) {
+				throw new Exception ("Invalid UserComment value, no charset found");
+			}
+
 			string charset = System.Text.Encoding.ASCII.GetString (raw_data, 0, 8);
-			string value = null;
 			System.Text.Encoding enc;
 
 			switch (charset) {
@@ -328,11 +373,17 @@ namespace FSpot.Tiff {
 				enc = System.Text.Encoding.ASCII;
 				break;
 			case "UNICODE\0":
-				enc = System.Text.Encoding.Unicode;
+			case "Unicode\0":
+				enc = new System.Text.UnicodeEncoding (! little, true);
 				break;
-			case "SJIS\0\0\0\0":
-				// FIXME I'm pretty sure this isn't actually the encoding name.
-				enc = System.Text.Encoding.GetEncoding ("SJIS");
+			case "JIS\0\0\0\0\0":
+				// FIXME this requires mono locale extras
+				try {
+					enc = System.Text.Encoding.GetEncoding ("euc-jp");
+				} catch {
+					System.Console.WriteLine ("missing jis0208 encoding");
+					enc = System.Text.Encoding.Default;
+				}
 				break;
 			case "\0\0\0\0\0\0\0\0":
 				// FIXME the spec says to use the local encoding in this case, we could probably
@@ -347,20 +398,104 @@ namespace FSpot.Tiff {
 			Charset = charset;
 			Value = enc.GetString (raw_data, 8, raw_data.Length - 8);
 		}
+
+		public byte [] GetBytes (bool is_little)
+		{
+			bool ascii = true;
+			string description = Value;
+			System.Text.Encoding enc;
+			string heading;
+
+			for (int i = 0; i < description.Length; i++) {
+				if (description [i] > 127) {
+					ascii = false;
+					break;
+				}
+			}
+
+			if (ascii) {
+				heading = "ASCII\0\0\0";
+				enc = new System.Text.ASCIIEncoding ();
+			} else {
+				heading = "Unicode\0";
+				enc = new System.Text.UnicodeEncoding (! is_little, true);
+			}
+			
+			int len = enc.GetByteCount (description);
+			byte [] data = new byte [len + heading.Length];
+			System.Text.Encoding.ASCII.GetBytes (heading, 0, heading.Length, data, 0);
+			enc.GetBytes (Value, 0, Value.Length, data, heading.Length);
+			
+			UserComment c = new UserComment (data, is_little);
+			System.Console.WriteLine ("old = \"{0}\" new = \"{1}\" heading = \"{2}\"", c.Value, description, heading);
+			return data;
+		}
 	}
 
 	public struct CFAPattern {
 		public ushort Rows;
 		public ushort Columns;
 		public byte [] Values;
-		
+
 		public CFAPattern (byte [] raw_data, bool little)
 		{
 			Columns = BitConverter.ToUInt16 (raw_data, 0, little);
 			Rows = BitConverter.ToUInt16 (raw_data, 2, little);
+
 			Values = new byte [Rows * Columns];
 			System.Array.Copy (raw_data, 4, Values, 0, Values.Length);
 		}
+
+		/* 
+		 * Note the Exif spec defines a CFA pattern tag that includes the row and column counts as shorts
+		 * inside the first four bytes of the entry.  The Tiff-EP standard define the CFARepeatPattern tag
+		 * that contains the row and column counts presumably since the Exif version wouldn't allow you to
+		 * alter the endian of the file without knowing the tag layout.
+		 */
+		
+		public CFAPattern (ushort rows, ushort cols, byte [] raw_data,  bool little)
+		{
+			Columns = rows;
+			Rows = cols;
+			Values = new byte [rows * cols];
+			System.Array.Copy (raw_data, 0, Values, 0, Values.Length);
+		}
+	}
+
+	public struct OECFTable {
+		public ushort Rows;
+		public ushort Columns;
+		public string [] Names;
+		public SRational [] Values;
+		
+		public OECFTable (byte [] raw_data, bool little)
+		{
+			Columns = BitConverter.ToUInt16 (raw_data, 0, little);
+			Rows = BitConverter.ToUInt16 (raw_data, 2, little);
+			Names = new string [Columns];
+			Values = new SRational [Columns * Rows];
+
+			int pos = 2;
+			int i;
+			int type_size = DirectoryEntry.GetTypeSize (EntryType.SRational);
+
+			for (i = 0; i < Names.Length; i++)
+				Names [i] = ReadString (raw_data, ref pos);
+			
+			for (i = 0; i < Values.Length; i++)
+				Values [i] = new SRational (raw_data, pos + i * type_size, little);
+			
+		}
+
+		public string ReadString (byte [] data, ref int pos)
+		{
+			int start = pos;
+			for (; pos < data.Length; pos++) {
+				if (data [pos] == 0)
+					break;
+			}	
+			return System.Text.Encoding.ASCII.GetString (data, start, pos - start);
+		}
 	}
 
 	public enum ExtraSamples {
@@ -369,6 +504,10 @@ namespace FSpot.Tiff {
 		UnassociatedAlpa = 2
 	}
 
+	public enum FileSource {
+		DCF = 3,
+	}
+
 	public enum PhotometricInterpretation : ushort {
 		WhiteIsZero = 0,
 		BlackIsZero = 1,
@@ -436,6 +575,16 @@ namespace FSpot.Tiff {
 		AutoBracket = 2
 	}
 
+	public enum CustomRendered : ushort {
+		Normal = 0,
+		Custom = 1
+	}
+
+
+	public enum SceneType {
+		DirectlyPhotographed = 1
+	}
+
 	public enum MeteringMode {
 		Uknown = 0,
 		Average = 1,
@@ -446,6 +595,44 @@ namespace FSpot.Tiff {
 		Partial = 6,
 	}
 
+	public enum SceneCaptureType : ushort {
+		Standard = 0,
+		Landscape = 1,
+		Portrait = 2,
+		NightScene = 3
+	}
+
+	public enum GainControl : ushort {
+		None = 0,
+		LowGainUp = 1,
+		HighGainUp = 2,
+		LowGainDown = 3,
+		HighGainDown = 4
+	}
+
+	public enum Contrast : ushort {
+		Normal = 0,
+		Soft = 1,
+		Hard = 2
+	}
+	
+	public enum Saturation : ushort {
+		Normal = 0,
+		Low = 1,
+		High = 2
+	}
+
+	public enum WhiteBalance : ushort {
+		Auto = 0,
+	        Manual = 1
+	}
+
+	public enum Sharpness : ushort {
+		Normal = 0,
+		Soft = 1,
+		Hard = 2
+	}
+
 	public enum LightSource {
 		Unknown = 0,
 		Daylight = 1,
@@ -470,8 +657,9 @@ namespace FSpot.Tiff {
 		OtherSource = 255
 	}
 
-	public enum ColorSpace : ushort {
-		sRGB = 1,
+	public enum ColorSpace {
+		StandardRGB = 1,  // sRGB
+		AdobeRGB = 2,
 		Uncalibrated = 0xffff
 	}
 
@@ -502,10 +690,9 @@ namespace FSpot.Tiff {
 
 	[System.Flags]
 	public enum NewSubfileType : uint {
-		SingleImage = 0,
-		ReducedResolutionFlag = 1,
-		PageOfMultipageFlag = 1 << 1,
-		TransparencyMaskFlag = 1 << 2
+		ReducedResolution = 1,
+		PageOfMultipage= 1 << 1,
+		TransparencyMask = 1 << 2
 	}
 
 	public enum EntryType {
@@ -616,9 +803,14 @@ namespace FSpot.Tiff {
 		private uint directory_offset;
 		public ImageDirectory Directory;
 
+                // false seems a safe default
+                public bool Distinct {
+                        get { return false; }
+                }
+
 		public Header (System.IO.Stream stream)
 		{
-			using (new Timer ("new Tiff.Header")) {
+			//using (new Timer ("new Tiff.Header")) {
 			byte [] data = new byte [8];
 			stream.Read (data, 0, data.Length);
 			if (data [0] == 'M' && data [1] == 'M')
@@ -629,7 +821,7 @@ namespace FSpot.Tiff {
 			ushort marker = BitConverter.ToUInt16 (data, 2, endian == Endian.Little);
 			switch (marker) {
 			case 42:
-				System.Console.WriteLine ("Found Standard Tiff Marker {0}", marker);
+				//System.Console.WriteLine ("Found Standard Tiff Marker {0}", marker);
 				break;
 			case 0x4f52:
 				System.Console.WriteLine ("Found Olympus Tiff Marker {0}", marker.ToString ("x"));
@@ -648,23 +840,27 @@ namespace FSpot.Tiff {
 			if (directory_offset < 8)
 				throw new System.Exception ("Invalid IFD0 Offset [" + directory_offset.ToString () + "]"); 
 			
-			//System.Console.WriteLine ("Reading First IFD");
+#if DEBUG_LOADER
+			System.Console.WriteLine ("Reading First IFD");
+#endif
 			Directory = new ImageDirectory (stream, directory_offset, endian); 
-			}
+			//}
 		}
 		
 		
 		public void Select (SemWeb.StatementSink sink)
 		{
-			using (new Timer ("Tiff.Header.Select")) {
+			//using (new Timer ("Tiff.Header.Select")) {
 				SelectDirectory (Directory, sink);
-			}
+			//}
 		}
 
-		private void SelectDirectory (ImageDirectory dir, StatementSink sink)
+		public void SelectDirectory (ImageDirectory dir, StatementSink sink)
 		{
 			foreach (DirectoryEntry e in dir.Entries) {
-				//System.Console.WriteLine ("{0}", e.Id);
+#if DEBUG_LOADER
+				System.Console.WriteLine ("{0}", e.Id);
+#endif
 				switch (e.Id) {
 				case TagId.IPTCNAA:
 					System.IO.Stream iptcstream = new System.IO.MemoryStream (e.RawData);
@@ -708,22 +904,30 @@ namespace FSpot.Tiff {
 					MetadataStore.AddLiteral (sink, "xmp:CreatorTool", e.ValueAsString [0]);
 					break;
 				case TagId.DateTime:
+					try {
+
 					MetadataStore.AddLiteral (sink, "xmp:ModifyDate", 
 								  e.ValueAsDate.ToString ("yyyy-MM-ddThh:mm:ss"));
+					} catch (System.Exception ex) {
+						System.Console.WriteLine (String.Format ("error parsing {0}\n{1}", e.ValueAsString[0], ex));
+					}
+
 					break;
 				case TagId.DateTimeOriginal:
 				case TagId.DateTimeDigitized:
 					// FIXME subsectime needs to be included in these values
 					// FIXME shouldn't DateTimeOriginal be xmp:CreateDate? the spec says no but wtf?
-					MetadataStore.AddLiteral (sink, "exif:" + e.Id.ToString (), 
-								  e.ValueAsDate.ToString ("yyyy-MM-ddThh:mm:ss"));
+					try {
+						MetadataStore.AddLiteral (sink, "exif:" + e.Id.ToString (), 
+									  e.ValueAsDate.ToString ("yyyy-MM-ddThh:mm:ss"));
+					} catch (System.Exception ex) {
+						System.Console.WriteLine (String.Format ("error parsing {0}\n{1}", e.ValueAsString[0], ex));
+					}
 					break;
-					//case TagId.Flash:
-					
 					//case TagId.SpatialFrequencyResponse
-				case TagId.CFAPattern:
+				case TagId.ExifCFAPattern:
 					CFAPattern pattern = new CFAPattern (e.RawData, e.IsLittle);
-					Entity empty = new Entity (null);
+					Entity empty = new BNode ();
 					Statement top = new Statement ("", 
 								       (Entity)MetadataStore.Namespaces.Resolve ("exif:" + e.Id.ToString ()),
 								       empty);
@@ -812,12 +1016,13 @@ namespace FSpot.Tiff {
 			}
 		}
 
-		public string Dump ()
+		public void Dump (string name)
 		{
-			System.Text.StringBuilder builder = new System.Text.StringBuilder ();
-			builder.Append (System.String.Format ("Header [{0}]\n", endian.ToString ()));
-			builder.Append (System.String.Format ("|-{0}", Directory.Dump2 ()));
-			return builder.ToString ();
+			ImageDirectory ifd = Directory;
+			for (int i = 0; ifd != null; i++) {
+				ifd.Dump (System.String.Format ("IFD[{0}]:", i));
+				ifd = ifd.NextDirectory;
+			}
 		}
 	}
 
@@ -845,7 +1050,7 @@ namespace FSpot.Tiff {
 			ReadHeader (stream);			
 			ReadEntries (stream);
 			ReadFooter (stream);
-
+			
 			LoadEntries (stream);
 			LoadNextDirectory (stream);
 		}
@@ -859,8 +1064,9 @@ namespace FSpot.Tiff {
 		protected virtual void ReadEntries (System.IO.Stream stream) 
 		{
 			num_entries = Converter.ReadUShort (stream, endian);
-			//System.Console.WriteLine ("reading {0} entries", num_entries);
-
+#if DEBUG_LOADER
+			System.Console.WriteLine ("reading {0} entries", num_entries);
+#endif			
 			entries = new System.Collections.ArrayList (num_entries);
 			int entry_length = num_entries * 12;
 			byte [] content = new byte [entry_length];
@@ -871,7 +1077,9 @@ namespace FSpot.Tiff {
 			for (int pos = 0; pos < entry_length; pos += 12) {
 				DirectoryEntry entry = EntryFactory.CreateEntry (this, content, pos, this.endian);
 				entries.Add (entry);		
-				//System.Console.WriteLine ("Added Entry {0} {1} - {2} * {3}", entry.Id.ToString (), entry.Id.ToString ("x"), entry.Type, entry.Count);
+#if DEBUG_LOADER
+				System.Console.WriteLine ("Added Entry {0} {1} - {2} * {3}", entry.Id.ToString (), entry.Id.ToString ("x"), entry.Type, entry.Count);
+#endif
 				if (entry.Id == TagId.NewSubfileType) {
 					
 				}
@@ -892,12 +1100,14 @@ namespace FSpot.Tiff {
 		
 		protected void LoadNextDirectory (System.IO.Stream stream)
 		{
-			//System.Console.WriteLine ("next_directory_offset = {0}", next_directory_offset);
+#if DEBUG_LOADER
+			System.Console.WriteLine ("start_position = {1} next_directory_offset = {0}", next_directory_offset, orig_position);
+#endif
 			next_directory = null;
 			try {
-				if (next_directory_offset != 0)
+				if (next_directory_offset != 0 && next_directory_offset != orig_position)
 					next_directory = new ImageDirectory (stream, next_directory_offset, this.endian);
-
+				
 			} catch (System.Exception e) {
 				//System.Console.WriteLine ("Error loading directory {0}", e.ToString ());
 				next_directory = null;
@@ -936,13 +1146,100 @@ namespace FSpot.Tiff {
 			return null;
 		}
 		
-		
-		public void Dump () 
+		public Cms.Profile GetProfile ()
 		{
-			//System.Console.WriteLine ("Directory Start");
+			Cms.ColorCIExyY whitepoint = new Cms.ColorCIExyY (0, 0, 0);
+			Cms.ColorCIExyYTriple primaries = new Cms.ColorCIExyYTriple (whitepoint, whitepoint, whitepoint);
+			Cms.GammaTable [] transfer = null;
+			int bits_per_sample = 8;
+			double gamma = 2.2;
+			
+			foreach (DirectoryEntry e in entries) {
+				switch (e.Id) {
+				case TagId.InterColorProfile:
+					try {
+						return new Cms.Profile (e.RawData);
+					} catch (System.Exception ex) {
+						System.Console.WriteLine (ex);
+					}
+					break;
+				case TagId.ColorSpace:
+					switch ((ColorSpace)e.ValueAsLong [0]) {
+					case ColorSpace.StandardRGB:
+						return Cms.Profile.CreateStandardRgb ();
+					case ColorSpace.AdobeRGB:
+						return Cms.Profile.CreateAlternateRgb ();
+					case ColorSpace.Uncalibrated:
+						System.Console.WriteLine ("Uncalibrated colorspace");
+						break;
+					}
+					break;
+
+				case TagId.WhitePoint:
+					Rational [] white = e.RationalValue;
+					whitepoint.x = white [0].Value;
+					whitepoint.y = white [1].Value;
+					whitepoint.Y = 1.0;
+					break;
+				case TagId.PrimaryChromaticities:
+					Rational [] colors = e.RationalValue;
+					primaries.Red.x = colors [0].Value;
+					primaries.Red.y = colors [1].Value;
+					primaries.Red.Y = 1.0;
+
+					primaries.Green.x = colors [2].Value;
+					primaries.Green.y = colors [3].Value;
+					primaries.Green.Y = 1.0;
+
+					primaries.Blue.x = colors [4].Value;
+					primaries.Blue.y = colors [5].Value;
+					primaries.Blue.Y = 1.0;
+					break;
+				case TagId.TransferFunction:
+					ushort [] trns = e.ShortValue;
+					ushort gamma_count = (ushort) (1 << bits_per_sample);
+					Cms.GammaTable [] tables = new Cms.GammaTable [3];
+					System.Console.WriteLine ("Parsing transfer function: count = {0}", trns.Length);
+
+					// FIXME we should use the TransferRange here
+					// FIXME we should use bits per sample here
+					for (int c = 0; c < 3; c++) {
+						tables [c] = new Cms.GammaTable (trns, c * gamma_count, gamma_count);
+					}
+
+					transfer = tables;
+					break;
+				case TagId.ExifIfdPointer:
+					SubdirectoryEntry exif = (SubdirectoryEntry) e;
+					DirectoryEntry ee = exif.Directory [0].Lookup ((int)TagId.Gamma);
+					
+					if (ee == null)
+						break;
+
+					Rational rgamma = ee.RationalValue [0];
+					gamma = rgamma.Value;
+					break;
+				}
+			}
+
+			if (transfer == null) {
+				Cms.GammaTable basic = new Cms.GammaTable (1 << bits_per_sample, gamma);
+				transfer = new Cms.GammaTable [] { basic, basic, basic };
+			}
+
+			// if we didn't get a white point or primaries, give up
+			if (whitepoint.Y != 1.0 || primaries.Red.Y != 1.0)
+				return null;
+				
+			return new Cms.Profile (whitepoint, primaries, transfer);
+		}
+
+		public void Dump (string name) 
+		{
+			System.Console.WriteLine ("Starting {0}", name);
 			foreach (DirectoryEntry e in this.Entries)
-				e.Dump ();
-			//System.Console.WriteLine ("End Directory");
+				e.Dump (name);
+			System.Console.WriteLine ("Ending {0}", name);
 		}
 		
 		public string Dump2 ()
@@ -965,11 +1262,6 @@ namespace FSpot.Tiff {
 		}
 	}
 	
-	public class DNGPrivateDirectory {
-		
-
-	}
-	
 	public class EntryFactory {
 		//public delegate DirectoryEntry ConstructorFunc (byte [], Endian endian);
 		//public static System.Collections.Hashtable ctors = new System.Collections.Hashtable ();
@@ -995,8 +1287,8 @@ namespace FSpot.Tiff {
 				//return new MakerNoteEntry (input, start, header_endian);
 				//case TagId.PimIfdPointer:
 				//return new 
-			case TagId.MakerNote:
-				return new MakerNoteEntry (input, start, header_endian);
+				//case TagId.MakerNote:
+				//return new MakerNoteEntry (input, start, header_endian);
 			}
 			
 			switch (type) {
@@ -1009,8 +1301,6 @@ namespace FSpot.Tiff {
 				return new LongEntry (input, start, header_endian);
 			case EntryType.Short:
 				return new ShortEntry (input, start, header_endian);
-			case EntryType.Ascii:
-				return new AsciiEntry (input, start, header_endian);
 			}
 
 			return new DirectoryEntry (input, start, header_endian);
@@ -1031,13 +1321,9 @@ namespace FSpot.Tiff {
 		public override void LoadExternal (System.IO.Stream stream)
 		{
 		}
-		
-		public override void Dump ()
-		{
-
-		}
 	}
 
+
 	public class SubdirectoryEntry : DirectoryEntry {
 		public uint directory_offset;
 		public ImageDirectory [] Directory;
@@ -1062,21 +1348,28 @@ namespace FSpot.Tiff {
 			base.LoadExternal (stream);
 
 			for (int i = 0; i <  entry_count; i++) {
-				directory_offset = BitConverter.ToUInt32 (raw_data, i * 4, endian == Endian.Little);
-				//System.Console.WriteLine ("Entering Subdirectory {0} at {1}", tagid.ToString (), directory_offset);
-				Directory [i] = new ImageDirectory (stream, directory_offset, endian);
-				//System.Console.WriteLine ("Leaving Subdirectory {0} at {1}", tagid.ToString (), directory_offset);
+				try {
+					directory_offset = BitConverter.ToUInt32 (raw_data, i * 4, endian == Endian.Little);
+					Directory [i] = new ImageDirectory (stream, directory_offset, endian);
+				} catch (System.Exception e) {
+					System.Console.WriteLine ("Error loading Subdirectory {0} at {2} of {3}bytes:\n{1}", 
+								  this.Id, e, directory_offset, stream.Length);
+				}
+					
 			}
 		}
 
-		public override void Dump ()
+		public override void Dump (string name)
 		{
 			for (int i = 0; i < GetEntryCount (); i++) {
-				//System.Console.WriteLine ("Entering Subdirectory {0}.{2} at {1}", tagid.ToString (), directory_offset, i);
-				if (Directory [i] != null)
-					Directory [i].Dump ();
-				
-				//System.Console.WriteLine ("Leaving Subdirectory {0}.{2} at {1}", tagid.ToString (), directory_offset, i);
+				string subdirname = System.String.Format ("{0}{1}[{2}]({3})]", name, tagid, i, directory_offset);
+
+				try {
+					if (Directory [i] != null)
+						Directory [i].Dump (subdirname);
+				} catch (System.Exception e) {
+					System.Console.WriteLine (e);
+				}
 			}
 		}
 	}
@@ -1103,19 +1396,11 @@ namespace FSpot.Tiff {
 		}
 	}
 	
-	public class AsciiEntry : DirectoryEntry {
-		public AsciiEntry (byte [] data, int offset, Endian endian) : base (data, offset, endian)
-		{
-			if (type != EntryType.Ascii)
-				throw new System.Exception (System.String.Format ("Invalid Settings At Birth {0}", tagid));
-		}
-	}
-	
 #if false
 	public class ImageLoader {
-		ushort width;
-		ushort length;
-		ushort [] bps;
+		int width;
+		int length;
+		int [] bps;
 		PhotometricInterpretation interpretation;
 		Compression compression;
 		uint [] offsets;
@@ -1127,8 +1412,7 @@ namespace FSpot.Tiff {
 		{
 			width = directory.Lookup (TagId.ImageWidth).ValueAsLong [0];
 			length = directory.Lookup (TagId.ImageLength).ValueAsLong [0];
-			
-			bps = ((ShortEntry)directory.Lookup (TagId.BitsPerSample)).Value;
+			bps = directory.Lookup (TagId.BitsPerSample).ValueAsLong;
 			
 			compression = (Compression) directory.Lookup (TagId.Compression).ValueAsLong [0];
 			interpretation = (PhotometricInterpretation) directory.Lookup (TagId.PhotometricInterpretation).ValueAsLong [0];
@@ -1191,8 +1475,7 @@ namespace FSpot.Tiff {
 			offset_origin = pos;
 		}
 
-		public uint Position
-		{
+		public uint Position {
 			get {
 				return offset_origin + data_offset;
 			}
@@ -1261,10 +1544,10 @@ namespace FSpot.Tiff {
 #if false
 			switch ((int)this.Id) {
 			case (int)TagId.NewSubfileType:
-				//System.Console.WriteLine ("XXXXXXXXXXXXXXXXXXXXX new NewSubFileType {0}", (NewSubfileType) this.ValueAsLong [0]);
+				System.Console.WriteLine ("XXXXXXXXXXXXXXXXXXXXX new NewSubFileType {0}", (NewSubfileType) this.ValueAsLong [0]);
 				break;
 			case (int)TagId.SubfileType:
-				//System.Console.WriteLine ("XXXXXXXXXXXXXXXXXXXXX new SubFileType {0}", (SubfileType) this.ValueAsLong [0]);
+				System.Console.WriteLine ("XXXXXXXXXXXXXXXXXXXXX new SubFileType {0}", (SubfileType) this.ValueAsLong [0]);
 				break;
 			case (int)TagId.Compression:
 				//System.Console.WriteLine ("XXXXXXXXXXXXXXXXXXXXX new Compression {0}", (Compression) this.ValueAsLong [0]);
@@ -1306,23 +1589,23 @@ namespace FSpot.Tiff {
 #endif
 		}
 
-		public virtual void Dump ()
+		public virtual void Dump (string name)
 		{
 			switch (this.Type) {
 			case EntryType.Short:
 			case EntryType.Long:
 				uint [] vals = this.ValueAsLong;
-				System.Console.Write ("{1}({2}) [{0}] (", vals.Length, this.Id, this.Type);
+				System.Console.Write ("{3}{1}({2}) [{0}] (", vals.Length, this.Id, this.Type, name);
 				for (int i = 0; i < System.Math.Min (15, vals.Length); i++) {
 					System.Console.Write (" {0}", vals [i]);
 				}
 				System.Console.WriteLine (")");
 				break;
 			case EntryType.Ascii:
-				System.Console.WriteLine ("{1}({2}) (\"{0}\")", this.StringValue, this.Id, this.Type);
+				System.Console.WriteLine ("{3}{1}({2}) (\"{0}\")", this.StringValue, this.Id, this.Type, name);
 				break;
 			default:
-				System.Console.WriteLine ("{1}({2}) [{0}]", this.Count, this.Id, this.Type);
+				System.Console.WriteLine ("{3}{1}({2}) [{0}]", this.Count, this.Id, this.Type, name);
 				break;
 			}
 		}
@@ -1406,15 +1689,13 @@ namespace FSpot.Tiff {
 			}
 		}
 
-		public byte [] RawData
-		{
+		public byte [] RawData {
 			get { 
 				return raw_data;
 			}
 		}
 
-		public string [] ValueAsString
-		{
+		public string [] ValueAsString {
 			get {
 				switch (this.Type) {
 				case EntryType.Short:
@@ -1434,6 +1715,8 @@ namespace FSpot.Tiff {
 					case TagId.FileSource:
 					case TagId.SceneType:
 						return ArrayToString (this.RawData);
+					case TagId.ComponentsConfiguration:
+						return ArrayToString (ValueAsLong);
 					default:
 						System.Console.WriteLine ("Cannot convert type \"{0}\" to string", Id);
 						break;
@@ -1441,7 +1724,6 @@ namespace FSpot.Tiff {
 					break;
 				case EntryType.Ascii:
 					return StringValue.Split ('\0');
-					break;
 				}
 				return null;
 			}
@@ -1456,8 +1738,7 @@ namespace FSpot.Tiff {
 			return vals;
 		}
 
-		public uint [] ValueAsLong
-		{
+		public uint [] ValueAsLong {
 			get {
 				uint [] data = new uint [this.Count];
 				for (int i = 0; i < this.Count; i++) {
@@ -1468,6 +1749,7 @@ namespace FSpot.Tiff {
 					case EntryType.Short:
 						data [i] = BitConverter.ToUInt16 (raw_data, i * GetTypeSize (), endian == Endian.Little);
 						break;
+					case EntryType.Undefined:
 					case EntryType.Byte:
 						data [i] = raw_data [i];
 						break;
@@ -1490,23 +1772,20 @@ namespace FSpot.Tiff {
 			}
 		}
 
-		public System.DateTime ValueAsDate
-		{
+		public System.DateTime ValueAsDate {
 			get {
 				return DirectoryEntry.DateTimeFromString (StringValue);
 			}
 		}
 
-		public string UserCommentValue
-		{
+		public string UserCommentValue {
 			get {
 				UserComment comment = new UserComment (raw_data, IsLittle);
 				return comment.Value;
 			}
 		}
 
-		public SRational [] SRationalValue
-		{
+		public SRational [] SRationalValue {
 			get {
 				Rational [] vals = RationalValue;
 				SRational [] data = new SRational [vals.Length];
@@ -1518,8 +1797,7 @@ namespace FSpot.Tiff {
 			}
 		}
 
-		public Rational [] RationalValue
-		{
+		public Rational [] RationalValue {
 			get {
 				uint [] vals = LongValue;
 				Rational [] data = new Rational [vals.Length / 2];
@@ -1532,8 +1810,7 @@ namespace FSpot.Tiff {
 			
 		}
 
-		public uint [] LongValue
-		{
+		public uint [] LongValue {
 			get {
 				uint [] data = new uint [raw_data.Length / 4];
 				for (int i = 0; i < raw_data.Length; i+= 4)
@@ -1543,8 +1820,7 @@ namespace FSpot.Tiff {
 			}
 		}
 
-		public ushort [] ShortValue
-		{
+		public ushort [] ShortValue {
 			get {
 				ushort [] data = new ushort [raw_data.Length];
 				for (int i = 0; i < raw_data.Length; i+= 2) {
@@ -1559,42 +1835,62 @@ namespace FSpot.Tiff {
 	public class TiffFile : ImageFile, SemWeb.StatementSource {
 		public Header Header;
 
+                // false seems a safe default
+                public bool Distinct {
+                        get { return false; }
+                }
+
 		public TiffFile (string path) : base (path)
 		{
 			try {
-				using (System.IO.Stream input = System.IO.File.OpenRead (path)) {
+				using (System.IO.Stream input = Open ()) {
 					this.Header = new Header (input);
 				}
-				
-				ImageDirectory directory = Header.Directory;
-				while (directory != null) {
-#if TEST_METADATA 
-					FSpot.MetadataStore store = new FSpot.MetadataStore ();
-					Select (store);
-					//store.Dump ();
-#endif					
-					///directory.Dump ();
-					directory = directory.NextDirectory;
-				}
-				
-				//System.Console.WriteLine (this.Header.Dump ());
+
+#if DEBUG_LOADER
+				Header.Dump (this.ToString () + ":");
+#endif
 			} catch (System.Exception e) {
 				System.Console.WriteLine (e.ToString ());
 			}
 		}
 
-		public void Select (SemWeb.StatementSink sink)
+		public TiffFile (Uri uri) : base (uri)
 		{
-			Header.Select (sink);
+			try {
+				using (System.IO.Stream input = Open ()) {
+					this.Header = new Header (input);
+				}
+
+#if DEBUG_LOADER
+				Header.Dump (this.ToString () + ":");
+#endif
+			} catch (System.Exception e) {
+				System.Console.WriteLine (e.ToString ());
+			}
 		}
 
-		public override System.DateTime Date
+		public virtual void Select (SemWeb.StatementSink sink)
 		{
+			Header.SelectDirectory (Header.Directory, sink);
+		}
+
+		public override System.DateTime Date {
 			get {
-				AsciiEntry e = (AsciiEntry)(this.Header.Directory.Lookup (TagId.DateTime));
-				
+				SubdirectoryEntry sub = (SubdirectoryEntry) this.Header.Directory.Lookup (TagId.ExifIfdPointer);
+				DirectoryEntry e;
+
+				if (sub != null) {
+					e = sub.Directory [0].Lookup (TagId.DateTimeOriginal);
+					
+					if (e != null)
+						return DirectoryEntry.DateTimeFromString (e.StringValue).ToUniversalTime ();
+				}
+
+				e = this.Header.Directory.Lookup (TagId.DateTime);
+
 				if (e != null)
-					return DirectoryEntry.DateTimeFromString (e.StringValue);
+					return DirectoryEntry.DateTimeFromString (e.StringValue).ToUniversalTime ();
 				else
 					return base.Date;
 			}
@@ -1602,7 +1898,7 @@ namespace FSpot.Tiff {
 		
 		public override System.IO.Stream PixbufStream ()
 		{
-			return null;
+			return Open ();
 		}
 
 		public override PixbufOrientation GetOrientation ()
@@ -1618,7 +1914,7 @@ namespace FSpot.Tiff {
 		{
 			uint offset = directory.Lookup (TagId.JPEGInterchangeFormat).ValueAsLong [0];
 			
-			System.IO.Stream file = System.IO.File.OpenRead (this.path);
+			System.IO.Stream file = Open ();
 			file.Position = offset;
 			return file;
 		}
@@ -1628,7 +1924,7 @@ namespace FSpot.Tiff {
 			uint offset = directory.Lookup (TagId.JPEGInterchangeFormat).ValueAsLong [0];
 			uint length = directory.Lookup (TagId.JPEGInterchangeFormatLength).ValueAsLong [0];
 			   
-			using (System.IO.Stream file = System.IO.File.OpenRead (this.path)) {
+			using (System.IO.Stream file = Open ()) {
 				file.Position = offset;
 				
 				byte [] data = new byte [32768];
@@ -1652,25 +1948,134 @@ namespace FSpot.Tiff {
 	}
 
 	public class DngFile : TiffFile {
-		public DngFile (string path) : base (path) {}
-
-		public override Gdk.Pixbuf Load (int width, int height)
+		public DngFile (string path) : base (path) 
 		{
-			return PixbufUtils.ScaleToMaxSize (this.Load (), width, height);
 		}
 
-		public override Gdk.Pixbuf Load ()
+		public DngFile (System.Uri uri) : base (uri) 
 		{
-			return DCRawFile.Load (this.path, null);
+		}
+
+		public override System.IO.Stream PixbufStream ()
+		{
+			try {
+				SubdirectoryEntry sub = (SubdirectoryEntry) Header.Directory.Lookup (TagId.SubIFDs);
+				ImageDirectory directory = sub.Directory [sub.Directory.Length - 1];
+
+				uint offset = directory.Lookup (TagId.StripOffsets).ValueAsLong [0];
+				System.IO.Stream file = Open ();
+				file.Position = offset;
+				return file;
+			} catch {
+				return DCRawFile.RawPixbufStream (uri);
+			}
+		}
+
+		public override void Select (SemWeb.StatementSink sink)
+		{
+			
+			/* this is just a sanity pass, if the first ifd is not a subfile use the normal
+			 * tiff path 
+			 */
+			DirectoryEntry e = Header.Directory.Lookup (TagId.NewSubfileType);
+			if (e == null) {
+				base.Select (sink);
+				return;
+			}
+
+			/*
+			 * Even though Ifd0 doesn't have the full resolution image
+			 * it would have the XMP data so we look for it
+			 */
+			e = Header.Directory.Lookup (TagId.XMP);
+			if (e != null) {
+				System.IO.Stream xmpstream = new System.IO.MemoryStream (e.RawData);
+				FSpot.Xmp.XmpFile xmp = new FSpot.Xmp.XmpFile (xmpstream);
+				xmp.Select (sink);
+			}
+
+			/* 
+			 * Ifd0 will also have the exif directory
+			 */
+			ImageDirectory dir = Header.Directory;
+			SubdirectoryEntry sub = (SubdirectoryEntry) dir.Lookup (TagId.ExifIfdPointer);
+			if (sub != null)
+				Header.SelectDirectory (sub.Directory [0], sink);
+			
+			/*
+			 * now we lookup subifd0 (we should probably scan the newsubfile types here)
+			 * and load the metadata we are interested in from it.
+			 */
+			sub = (SubdirectoryEntry) Header.Directory.Lookup (TagId.SubIFDs);	
+
+			int i = 0;
+			do {
+				uint dirtype = e.ValueAsLong [0];
+				if (dirtype == 0) {
+					Header.SelectDirectory (dir, sink);
+					break;
+				}
+					
+				if (sub == null)
+					break;
+
+				dir = sub.Directory [i];
+				e = dir.Lookup (TagId.NewSubfileType);
+				i++;
+			} while (i < sub.Directory.Length);
+
+			
 		}
 	}	
 	
 	public class NefFile : TiffFile, IThumbnailContainer {
-		public NefFile (string path) : base (path) {}
+		public NefFile (string path) : base (path) 
+		{
+		}
+
+		public NefFile (Uri uri) : base (uri)
+		{
+		}
+
+		public override void Select (SemWeb.StatementSink sink)
+		{
+			DirectoryEntry e = Header.Directory.Lookup (TagId.NewSubfileType);
+
+			if (e == null) {
+				base.Select (sink);
+				return;
+			}
+
+			ImageDirectory dir = Header.Directory;
+			SubdirectoryEntry sub = (SubdirectoryEntry) dir.Lookup (TagId.ExifIfdPointer);
+			
+			if (sub != null)
+				Header.SelectDirectory (sub.Directory [0], sink);
+			
+			sub = (SubdirectoryEntry) Header.Directory.Lookup (TagId.SubIFDs);	
+
+			int i = 0;
+			do {
+				uint dirtype = e.ValueAsLong [0];
+				if (dirtype == 0) {
+					Header.SelectDirectory (dir, sink);
+					break;
+				}
+					
+				if (sub == null)
+					break;
+
+				dir = sub.Directory [i];
+				e = dir.Lookup (TagId.NewSubfileType);
+				i++;
+			} while (i < sub.Directory.Length);
+		}
 
 		public Gdk.Pixbuf GetEmbeddedThumbnail ()
 		{
-			return TransformAndDispose (new Gdk.Pixbuf (path));
+			using (System.IO.Stream stream = Open ()) {
+				return TransformAndDispose (new Gdk.Pixbuf (stream));
+			}
 		}
 
 		public override System.IO.Stream PixbufStream ()
@@ -1680,44 +2085,28 @@ namespace FSpot.Tiff {
 				ImageDirectory jpeg_directory = sub.Directory [0];
 				return LookupJpegSubstream (jpeg_directory);
 			} catch (System.Exception e) {
-				return null;
+				return DCRawFile.RawPixbufStream (uri);
 			}
 		}
-		
-		public override Gdk.Pixbuf Load () 
-		{
-			Gdk.Pixbuf pixbuf = null;
-			System.Console.WriteLine ("starting load");
-			
-			try {
-				SubdirectoryEntry sub = (SubdirectoryEntry) Header.Directory.Lookup (TagId.SubIFDs);
-				ImageDirectory jpeg_directory = sub.Directory [0];
-				
-				pixbuf = LoadJpegInterchangeFormat (jpeg_directory);
-			} catch (System.Exception e) {
-				System.Console.WriteLine (e);
-				pixbuf = null;
-			}
-
-			if (pixbuf == null)
-				return DCRawFile.Load (this.Path, null);
-			
-			return TransformAndDispose (pixbuf);
-		}
-
-		public override Gdk.Pixbuf Load (int width, int height)
-		{
-			return PixbufUtils.ScaleToMaxSize (this.Load (), width, height);
-		}
 	}
 		
 
 	public class Cr2File : TiffFile, IThumbnailContainer {
-
 		public Cr2File (string path) : base (path) 
 		{
 		}
-		
+
+		public Cr2File (Uri uri) : base (uri)
+		{
+		}
+
+		/*
+		public override PixbufOrientation GetOrientation ()
+		{
+			return PixbufOrientation.TopLeft;
+		}
+		*/
+
 		public Gdk.Pixbuf GetEmbeddedThumbnail ()
 		{
 			ImageDirectory directory;
@@ -1725,34 +2114,14 @@ namespace FSpot.Tiff {
 			return TransformAndDispose (LoadJpegInterchangeFormat (directory));
 		}
 
-		public override Gdk.Pixbuf Load ()
+
+		public override System.IO.Stream PixbufStream ()
 		{
-#if false
-			return GetEmbeddedThumbnail ();
-#else
-			return DCRawFile.Load (this.Path, null);
-#endif
+			uint offset = Header.Directory.Lookup (TagId.StripOffsets).ValueAsLong [0];
+			System.IO.Stream file = Open ();
+			file.Position = offset;
+			return file;
 		}
-
-		public override Gdk.Pixbuf Load (int width, int height)
-		{
-			return PixbufUtils.ScaleToMaxSize (this.Load (), width, height);
-		}
-
-
-		public override System.DateTime Date
-		{
-			get {
-				SubdirectoryEntry sub = (SubdirectoryEntry) this.Header.Directory.Lookup (TagId.ExifIfdPointer);
-				AsciiEntry e = (AsciiEntry)(sub.Directory [0].Lookup (TagId.DateTimeOriginal));
-				
-				if (e != null)
-					return DirectoryEntry.DateTimeFromString (e.StringValue);
-				else
-					return base.Date;
-			}
-		}
-
 	}
 }
 
