@@ -4,8 +4,8 @@ using System;
 using System.IO;
 using System.Runtime.InteropServices;
 using System.Text;
-
-
+using System.Collections;
+using Mono.Unix;
 
 namespace FSpot 
 {
@@ -24,6 +24,8 @@ public class Driver {
 		try {
 			foreach (string arg in args) {
 				if (arg == "--help") {
+					Catalog.Init ("f-spot", Defines.LOCALE_DIR);
+
 					System.Console.WriteLine ("Usage f-spot [OPTION. ..]\n");
 					System.Console.WriteLine ("  --import [uri]\t\t\timport from the given uri");
 					System.Console.WriteLine ("  --view <file>\t\t\t\tview a file or directory ");
@@ -38,6 +40,8 @@ public class Driver {
 							       Modules.UI, args);
 					return;
 				} else if (arg == "--slideshow") {
+					Catalog.Init ("f-spot", Defines.LOCALE_DIR);
+						
 					program = new Program (Defines.PACKAGE, 
 							       Defines.VERSION, 
 						       Modules.UI, args);
@@ -57,37 +61,46 @@ public class Driver {
 					       Defines.VERSION, 
 					       Modules.UI, args);		
 			
-			try {
-				control = Core.FindInstance ();
-				System.Console.WriteLine ("Found active FSpot server: {0}", control);
-				program = null;
-			} catch (System.Exception e) { 
-				System.Console.WriteLine ("Starting new FSpot server");
-			}
-			
-			if (control == null) {
-				Core core = null;
-				
-				Gnome.Vfs.Vfs.Initialize ();
-				StockIcons.Initialize ();
-				
-				Mono.Posix.Catalog.Init ("f-spot", Defines.LOCALE_DIR);
-				Gtk.Window.DefaultIconList = new Gdk.Pixbuf [] {PixbufUtils.LoadFromAssembly ("f-spot-logo.png")};
-				
-			// FIXME: Error checking is non-existant here...
-				
-				core = new Core ();
-				
+			bool create = true;
+			while (control == null) {
 				try {
-					core.RegisterServer ();
+					control = Core.FindInstance ();
+					System.Console.WriteLine ("Found active FSpot server: {0}", control);
+					program = null;
+				} catch (System.Exception e) { 
+					Core.AssertOwnership ();
+					System.Console.WriteLine ("Starting new FSpot server");
+				}
+			
+				Core core = null;
+				try {
+					if (control == null && create) {
+						create = false;
+						Gnome.Vfs.Vfs.Initialize ();
+						StockIcons.Initialize ();
+						
+						Catalog.Init ("f-spot", Defines.LOCALE_DIR);
+						Gtk.Window.DefaultIconList = new Gdk.Pixbuf [] {PixbufUtils.LoadFromAssembly ("f-spot-logo.png")};
+						
+						core = new Core ();
+						core.RegisterServer ();
+						
+						// FIXME: Error checking is non-existant here...
+						
+						empty = Core.Database.Empty;
+						control = core;
+					}
 				} catch (System.Exception e) {
-					System.Console.WriteLine (e.ToString ());
+					System.Console.WriteLine ("XXXXX\n{0}\nXXXXX", e);
+					control = null;
+
+					if (core != null)
+						core.UnregisterServer ();
 				}
-				
-				empty = Core.Database.Empty;
-				control = core;
 			}
 			
+			
+			UriList list = new UriList ();
 			for (int i = 0; i < args.Length; i++) {
 				switch (args [i]) {
 				case "--shutdown":
@@ -100,9 +113,12 @@ public class Driver {
 					import = true;
 					break;
 				case "--view":
-					if (++i < args.Length)
-						control.View (args [i]);
+					while (++i < args.Length)
+						list.AddUnknown (args [i]);
 					
+					if (list.Count > 0)
+						control.View (list.ToString ());
+
 					view_only = true;
 					break;
 				}
@@ -111,9 +127,11 @@ public class Driver {
 			if (empty && !import)
 				control.Import (null);
 			
-			if (import || !view_only)
+			if (import || !view_only) {
 				control.Organize ();
-			
+				Gdk.Global.NotifyStartupComplete ();
+			}			
+
 			if (program != null)
 				program.Run ();
 			
@@ -142,6 +160,8 @@ public class Driver {
 			}
 		} catch (DllNotFoundException de) {
 			/* noop */
+		} catch (EntryPointNotFoundException de) {
+		    	/* noop */
 		}
 	}
 }
